@@ -2,7 +2,7 @@
 #
 #     written by G. S. Marzot (marz@users.sourceforge.net)
 #
-#     Copyright (c) 2006-2009 SPARTA, Inc.  All rights reserved.
+#     Copyright (c) 2006-2011 SPARTA, Inc.  All rights reserved.
 #
 #     Copyright (c) 2006-2009 G. S. Marzot. All rights reserved.
 #
@@ -87,10 +87,10 @@ our @EXPORT = qw(
  VAL_NOT_IMPLEMENTED VAL_NO_ERROR VAL_NO_PERMISSION VAL_NO_POLICY
  VAL_OUT_OF_MEMORY VAL_POL_GOPT_DISABLE VAL_POL_GOPT_ENABLE
  VAL_POL_GOPT_OVERRIDE VAL_PINSECURE VAL_QFLAGS_ANY
- VAL_QFLAGS_USERMASK VAL_QFLAGS_STATE VAL_QFLAGS_CACHE_MASK
+ VAL_QFLAGS_USERMASK VAL_QFLAGS_CACHE_MASK
  VAL_QUERY_NO_AC_DETAIL VAL_QUERY_NO_EDNS0 VAL_QUERY_DONT_VALIDATE 
  VAL_QUERY_GLUE_REQUEST VAL_QUERY_USING_DLV VAL_QUERY_NO_DLV 
- VAL_RESOURCE_UNAVAILABLE VAL_SUCCESS
+ VAL_RESOURCE_UNAVAILABLE VAL_SUCCESS VAL_QUERY_REFRESH_QCACHE
  VAL_TRUSTED_ANSWER VAL_UNTRUSTED_ANSWER
  VAL_UNTRUSTED_ZONE VAL_VALIDATED_ANSWER 
  ZONE_USE_NOTHING ZONE_USE_TSIG ns_t_dlv ns_t_dnskey ns_t_ds ns_t_nsec
