@@ -4,7 +4,7 @@
 # can do whatever you want with this stuff. If we meet some day, and you think
 # this stuff is worth it, you can buy me a beer in return.   Anton Berezin
 # ----------------------------------------------------------------------------
-# Copyright (c) 2005-2010 SPARTA, Inc.
+# Copyright (c) 2005-2011 SPARTA, Inc.
 # All rights reserved.
 #  
 # Redistribution and use in source and binary forms, with or without
@@ -33,7 +33,7 @@
 # OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 # ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #
-# $Id: Fast.pm 4996 2010-01-24 18:14:06Z tewok $
+# $Id: Fast.pm 5403 2011-02-15 00:25:58Z hardaker $
 #
 package Net::DNS::ZoneFile::Fast;
 # documentation at the __END__ of the file
@@ -46,7 +46,7 @@ use Net::DNS;
 use Net::DNS::RR;
 use MIME::Base64;
 
-$VERSION = '1.12';
+$VERSION = '1.15';
 
 my $MAXIMUM_TTL = 0x7fffffff;
 
@@ -68,6 +68,7 @@ my @zone;
 my $soa;
 my $rrsig;
 my $sshfp;
+my $key;
 my $dnskey;
 my $ds;
 my $on_error;
@@ -280,7 +281,7 @@ sub parse_line
 	  				$offset ||= 0;
 	  				$width  ||= 0;
 	  				$base   ||= 'd';
-	  				sprintf "%$width$base", $offset + $from;
+	  				sprintf "%0$width$base", $offset + $from;
 	  			}xge;
 	  		$parse->();
 	  		$from++;
@@ -618,7 +619,7 @@ sub parse_line
 			fptype => $2,
 		       };
 	      $parse = \&parse_sshfp;
-	  } elsif (/\G(\d+)\s+(\d+)\s+(.*)$pat_skip$/gc) {
+	  } elsif (/\G(\d+)\s+(\d+)\s+([a-zA-Z0-9]+)$pat_skip$/gc) {
 	      push @zone, {
 			   Line    => $ln,
 			   name    => $domain,
@@ -730,6 +731,35 @@ sub parse_line
 	  } else {
 	      error("bad SRV data");
 	  }
+      } elsif (/\G(key)[ \t]+/igc) {
+	  if (!/\G(\d+)\s+(\d+)\s+(\d+)\s+/gc) {
+	      error("bad KEY data 1");
+	  }
+	  $dnskey = {
+		     first     => 1,
+		     Line      => $ln,
+		     name      => $domain,
+		     ttl       => $ttl,
+		     class     => "IN",
+		     type      => "KEY",
+		     flags     => $1,
+		     protocol  => $2,
+		     algorithm => $3
+		    };
+	  if (/\G\(\s*$/gc) {
+	      # multi-line
+	      $parse = \&parse_dnskey;
+	  } elsif (/\G(.*\S)\s*$/) {
+	      # single-line
+	      $dnskey->{'key'} .= $1;
+	      $dnskey->{'key'} =~ s/\s//g;
+	      $dnskey->{'keybin'} = decode_base64($dnskey->{'key'});
+	      push @zone, $dnskey;
+	      $dnskey = undef;
+	  } else {
+	      error("bad KEY data 2");
+	  }
+
       } elsif (/\G(rrsig)[ \t]+/igc) {
 	  if (!/\G(\w+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)\s+/gc) {
 	      error("bad RRSIG data 1");
@@ -780,7 +810,7 @@ sub parse_line
 	  if (/\G\(\s*$/gc) {
 	      # multi-line
 	      $parse = \&parse_dnskey;
-	  } elsif (/\G(.*\S)\s*$/) {
+	  } elsif (/\G([\sA-Za-z0-9\+\/=]+).*$/) {
 	      # single-line
 	      $dnskey->{'key'} .= $1;
 	      $dnskey->{'key'} =~ s/\s//g;
@@ -811,6 +841,7 @@ sub parse_line
 	      # single line
 	      $ds->{'digest'} .= $1;
 	      $ds->{'digest'} = lc($ds->{'digest'});
+	      $ds->{'digest'} =~ s/\s//g;
 	      $ds->{'digestbin'} = pack("H*", $ds->{'digest'});
 	      push @zone, $ds;
 	      $ds = undef;
@@ -818,7 +849,7 @@ sub parse_line
 	      error("bad DS data");
 	  }
       } elsif (/\G(nsec)[ \t]+/igc) {
-	  if (/\G\s*($pat_maybefullnameorroot)\s+(.*)$pat_skip$/gc) {
+	  if (/\G\s*($pat_maybefullnameorroot)\s+(.*?)$pat_skip$/gc) {
 	      # XXX: set the typebm field ourselves?
 	      my ($nxtdname, $typelist) = ($1, $2);
 	      $typelist = join(" ",sort split(/\s+/,$typelist));
@@ -840,7 +871,7 @@ sub parse_line
       } elsif (/\G(nsec3)[ \t]+/igc) {
 	  error ("You are missing required modules for NSEC3 support")
 	    if (!$nsec3capable);
-	  if (/\G\s*(\d+)\s+(\d+)\s+(\d+)\s+([-0-9A-Fa-f]+)\s+($pat_maybefullname)\s+(.*)$pat_skip$/gc) {
+	  if (/\G\s*(\d+)\s+(\d+)\s+(\d+)\s+([-0-9A-Fa-f]+)\s+($pat_maybefullname)\s+(.*?)$pat_skip$/gc) {
 	      # XXX: set the typebm field ourselves?
 	      my ($alg, $flags, $iters, $salt, $nxthash, $typelist) =
 		($1, $2, $3, $4, $5, $6);
@@ -1180,6 +1211,8 @@ sub parse_soa_number
 	  }
 	  delete $soa->{nextkey};
 	  delete $soa->{breakable};
+          $soa->{mname} .= $origin unless ($soa->{mname} =~ /\.$/);
+          $soa->{rname} .= $origin unless ($soa->{rname} =~ /\.$/);
 	  $soa->{mname} =~ s/\.$//;
 	  $soa->{rname} =~ s/\.$//;
 	  $soa->{Lines} = $ln - $soa->{Line} + 1;
@@ -1433,7 +1466,7 @@ Copyright 2003 by Anton Berezin and catpipe Systems ApS
 
   Anton Berezin
 
-Copyright (c) 2004-2010 SPARTA, Inc.
+Copyright (c) 2004-2011 SPARTA, Inc.
   All rights reserved.
    
   Redistribution and use in source and binary forms, with or without
