@@ -1,5 +1,5 @@
 #
-# Copyright 2006-2010 SPARTA, Inc.  All rights reserved.  See the COPYING
+# Copyright 2006-2011 SPARTA, Inc.  All rights reserved.  See the COPYING
 # file distributed with this software for details
 #
 # DNSSEC Tools
@@ -25,14 +25,15 @@ our @EXPORT = qw(
 		   dnssec_tools_defnames
 		);
 
-our $VERSION = "1.8";
+our $VERSION = "1.9";
+our $MODULE_VERSION = "1.9.0";
 
 my $installdir = getprefixdir() . "/bin"; # DNSSEC-Tools installation directory.
 
 my %defaults =
 (
 	"admin-email"	   => "root",		# Admin's email address.
-	"archivedir"	   => getconfdir() . "/key-archive",
+	"archivedir"	   => getprefixdir() . "/var/key-archive",
 	"algorithm"	   => "rsasha1",	# Encryption algorithm.
 	"enddate"	   => "+2592000",	# Zone life, in seconds.
 	"entropy_msg"	   => 1,		# Display entropy message flag.
@@ -49,7 +50,9 @@ my %defaults =
 	"roll_loadzone"	   => 1,		# Zone-reloading flag.
 	"roll_logfile"	   => makelocalstatedir() . "/log.rollerd",
 	"roll_loglevel"	   => "phase",		# Rollerd's logging level.
+	"roll_phasemsg"	   => "long",		# Rollerd's phase logmsg length.
 	"roll_sleeptime"   => 3600,		# Rollerd's sleep time.
+	"log_tz"	   => "gmt",		# Log message timezone.
 	"savekeys"	   => 1,		# Save/delete old keys flag.
 	"tacontact" 	   => "",
 	"tatmpdir" 	   => "/tmp/dnssec-tools/trustman",
@@ -69,7 +72,7 @@ my %defaults =
 	"zsklength"	   => 1024,		# Length of ZSK key.
 	"zsklife"	   => 604800,		# Lifespan of ZSK key.
 	"usensec3"         => "no",             # Use NSEC3 by default
-	"nsec3iter"	   => 100,              # default NSEC3 iteratations
+	"nsec3iter"	   => 100,              # default NSEC3 iterations
 	"nsec3salt"	   => "random:64",      # default NSEC3 salt
 	"nsec3optout"	   => "no",             # NSEC3 opt-out default
 	"blinkenlights"	   => "$installdir/blinkenlights",
@@ -247,7 +250,7 @@ concept of a lifespan.  This is measured in seconds.
 
 =item B<revperiod>
 
-This default holds the default revocation periodof a KSK key. This is
+This default holds the default revocation period of a KSK key. This is
 the minimum period of time a revoked KSK is required to remain in the
 signing set so that it is properly observed by resolvers.  This is
 measured in seconds.
@@ -260,6 +263,10 @@ This default is the maximum lifespan of a key.
 
 This default is the minimum lifespan of a key.
 
+=item B<log_tz>
+
+This default is the timezone to be used in log-message timestamps.
+
 =item B<random>
 
 This default holds the default random number generator device.
@@ -481,7 +488,7 @@ This default holds the path to the DNSSEC-Tools B<zonesigner> program.
 
 =head1 COPYRIGHT
 
-Copyright 2006-2010 SPARTA, Inc.  All rights reserved.
+Copyright 2006-2011 SPARTA, Inc.  All rights reserved.
 See the COPYING file included with the DNSSEC-Tools package for details.
 
 =head1 AUTHOR
