@@ -1,5 +1,5 @@
 #
-# Copyright 2006-2010 SPARTA, Inc.  All rights reserved.  See the COPYING
+# Copyright 2006-2011 SPARTA, Inc.  All rights reserved.  See the COPYING
 # file distributed with this software for details
 #
 # DNSSEC Tools
@@ -22,7 +22,8 @@ use Net::DNS::SEC::Tools::defaults;
 use Net::DNS::SEC::Tools::keyrec;
 use Net::DNS::SEC::Tools::rollrec;
 
-our $VERSION = "1.8";
+our $VERSION = "1.9";
+our $MODULE_VERSION = "1.9.0";
 
 our @ISA = qw(Exporter);
 
@@ -108,6 +109,11 @@ sub dt_adminmail
 	#
 	return(0) if($sendto eq "");
 
+	#
+	# If things are configured to not send email, we'll return success.
+	#
+	return(1) if($sendto eq "nomail");
+
 	#
 	# Create the message object.
 	#
@@ -120,22 +126,23 @@ sub dt_adminmail
 	$msg->subject($subject);
 
 	#
-	# Open the 'connection' and add the message body.
+	# Open the "connection" and add the message body.
 	#
 	push @mailargs, $dtconf{'mailer-type'}
-		if (defined($dtconf{'mailer-type'}));
+		if(defined($dtconf{'mailer-type'}));
 	push @mailargs, 'smtp'
-		if (!defined($dtconf{'mailer-type'}) &&
-			defined($dtconf{'mailer-server'}));
+		if(!defined($dtconf{'mailer-type'}) &&
+		    defined($dtconf{'mailer-server'}));
 	push @mailargs, Server => $dtconf{'mailer-server'}
-		if (defined($dtconf{'mailer-server'}));
-	$mh = $msg->open(@mailargs);
+		if(defined($dtconf{'mailer-server'}));
+
+	eval { $mh = $msg->open(@mailargs); }; return(0) if $@;
 	print $mh $msgbody . "\n";
 
 	#
 	# Complete the message and send it.
 	#
-	$mh->close;
+	eval { $mh->close; }; return(0) if $@;
 	return(1);
 }
 
@@ -280,6 +287,8 @@ The I<msgbody> parameter is the body of the mail message.
 A third parameter, I<recipient>, may be given to specify the message's
 recipient.  If this is not given, then the recipient will be taken from
 the I<admin-email> record of the DNSSEC-Tools configuration file.
+If I<recipient> is "nomail", then no message will be sent and success
+will be returned.
 
 Return values:
 
@@ -303,7 +312,7 @@ mail.  If I<mailer-server> is set to a defined value but I<mailer-type> is not,
 =item I<mailer-server>
 
 The server, if I<admin-mail> is set to I<smtp>, that the mail should
-be delievered to.
+be delivered to.
 
 =item I<dt_cmdpath(command)>
 
@@ -343,7 +352,7 @@ Return values:
 
 =head1 COPYRIGHT
 
-Copyright 2006-2010 SPARTA, Inc.  All rights reserved.
+Copyright 2006-2011 SPARTA, Inc.  All rights reserved.
 See the COPYING file included with the DNSSEC-Tools package for details.
 
 =head1 AUTHOR
