@@ -1,5 +1,5 @@
 #
-# Copyright 2005-2010 SPARTA, Inc.  All rights reserved.  See the COPYING
+# Copyright 2005-2011 SPARTA, Inc.  All rights reserved.  See the COPYING
 # file distributed with this software for details
 #
 # DNSSEC Tools
@@ -96,7 +96,8 @@ use Net::DNS::SEC::Tools::conf;
 use Net::DNS::SEC::Tools::defaults;
 use Net::DNS::SEC::Tools::rolllog;
 
-our $VERSION = "1.8";
+our $VERSION = "1.9";
+our $MODULE_VERSION = "1.9.0";
 
 our @ISA = qw(Exporter);
 
@@ -109,6 +110,7 @@ our @EXPORT = qw(
 		 rollmgr_idfile
 		 rollmgr_set_idfile
 		 rollmgr_loadzone
+		 rollmgr_phasemsg
 		 rollmgr_rmid
 		 rollmgr_running
 		 rollmgr_saveid
@@ -132,9 +134,13 @@ our @EXPORT = qw(
 			 ROLLCMD_LOGFILE
 			 ROLLCMD_LOGLEVEL
 			 ROLLCMD_LOGMSG
+			 ROLLCMD_LOGTZ
+			 ROLLCMD_PHASEMSG
 			 ROLLCMD_ROLLALL
+			 ROLLCMD_ROLLALLZSKS
 			 ROLLCMD_ROLLKSK
 			 ROLLCMD_ROLLREC
+			 ROLLCMD_ROLLZONE
 			 ROLLCMD_ROLLZSK
 			 ROLLCMD_RUNQUEUE
 			 ROLLCMD_SHUTDOWN
@@ -151,6 +157,7 @@ our @EXPORT = qw(
 			 ROLLCMD_RC_BADFILE
 			 ROLLCMD_RC_BADSLEEP
 			 ROLLCMD_RC_BADROLLREC
+			 ROLLCMD_RC_BADTZ
 			 ROLLCMD_RC_DISPLAY
 			 ROLLCMD_RC_RRFOPEN
 			 ROLLCMD_RC_NOZONES
@@ -197,20 +204,22 @@ my $ROLLCMD_RC_BADLEVEL		= 1;
 my $ROLLCMD_RC_BADFILE		= 2;
 my $ROLLCMD_RC_BADSLEEP	 	= 3;
 my $ROLLCMD_RC_BADROLLREC	= 4;
-my $ROLLCMD_RC_RRFOPEN		= 5;
-my $ROLLCMD_RC_NOZONES		= 6;
-my $ROLLCMD_RC_BADZONE		= 7;
-my $ROLLCMD_RC_BADZONEDATA	= 8;
-my $ROLLCMD_RC_DISPLAY		= 9;
-my $ROLLCMD_RC_KSKROLL		= 10;
-my $ROLLCMD_RC_ZSKROLL		= 11;
-my $ROLLCMD_RC_NOARGS		= 12;
+my $ROLLCMD_RC_BADTZ		= 5;
+my $ROLLCMD_RC_RRFOPEN		= 6;
+my $ROLLCMD_RC_NOZONES		= 7;
+my $ROLLCMD_RC_BADZONE		= 8;
+my $ROLLCMD_RC_BADZONEDATA	= 9;
+my $ROLLCMD_RC_DISPLAY		= 10;
+my $ROLLCMD_RC_KSKROLL		= 11;
+my $ROLLCMD_RC_ZSKROLL		= 12;
+my $ROLLCMD_RC_NOARGS		= 13;
 
 sub ROLLCMD_RC_OKAY		{ return($ROLLCMD_RC_OKAY);		};
 sub ROLLCMD_RC_BADLEVEL		{ return($ROLLCMD_RC_BADLEVEL);		};
 sub ROLLCMD_RC_BADFILE		{ return($ROLLCMD_RC_BADFILE);		};
 sub ROLLCMD_RC_BADSLEEP		{ return($ROLLCMD_RC_BADSLEEP);		};
 sub ROLLCMD_RC_BADROLLREC	{ return($ROLLCMD_RC_BADROLLREC);	};
+sub ROLLCMD_RC_BADTZ		{ return($ROLLCMD_RC_BADTZ);		};
 sub ROLLCMD_RC_RRFOPEN		{ return($ROLLCMD_RC_RRFOPEN);		};
 sub ROLLCMD_RC_NOZONES		{ return($ROLLCMD_RC_NOZONES);		};
 sub ROLLCMD_RC_BADZONE		{ return($ROLLCMD_RC_BADZONE);		};
@@ -231,9 +240,13 @@ my $ROLLCMD_GETSTATUS	= "rollcmd_getstatus";
 my $ROLLCMD_LOGFILE	= "rollcmd_logfile";
 my $ROLLCMD_LOGLEVEL	= "rollcmd_loglevel";
 my $ROLLCMD_LOGMSG	= "rollcmd_logmsg";
-my $ROLLCMD_ROLLALL	= "rollcmd_rollallzsks";
+my $ROLLCMD_LOGTZ	= "rollcmd_logtz";
+my $ROLLCMD_PHASEMSG	= "rollcmd_phasemsg";
+my $ROLLCMD_ROLLALL	= "rollcmd_rollall";
+my $ROLLCMD_ROLLALLZSKS	= "rollcmd_rollallzsks";
 my $ROLLCMD_ROLLKSK	= "rollcmd_rollksk";
 my $ROLLCMD_ROLLREC	= "rollcmd_rollrec";
+my $ROLLCMD_ROLLZONE	= "rollcmd_rollzone";
 my $ROLLCMD_ROLLZSK	= "rollcmd_rollzsk";
 my $ROLLCMD_RUNQUEUE	= "rollcmd_runqueue";
 my $ROLLCMD_SHUTDOWN	= "rollcmd_shutdown";
@@ -252,9 +265,13 @@ sub ROLLCMD_GETSTATUS		{ return($ROLLCMD_GETSTATUS);	};
 sub ROLLCMD_LOGFILE		{ return($ROLLCMD_LOGFILE);	};
 sub ROLLCMD_LOGLEVEL		{ return($ROLLCMD_LOGLEVEL);	};
 sub ROLLCMD_LOGMSG		{ return($ROLLCMD_LOGMSG);	};
+sub ROLLCMD_LOGTZ		{ return($ROLLCMD_LOGTZ);	};
+sub ROLLCMD_PHASEMSG		{ return($ROLLCMD_PHASEMSG);	};
 sub ROLLCMD_ROLLALL		{ return($ROLLCMD_ROLLALL);	};
+sub ROLLCMD_ROLLALLZSKS		{ return($ROLLCMD_ROLLALLZSKS);	};
 sub ROLLCMD_ROLLKSK		{ return($ROLLCMD_ROLLKSK);	};
 sub ROLLCMD_ROLLREC		{ return($ROLLCMD_ROLLREC);	};
+sub ROLLCMD_ROLLZONE		{ return($ROLLCMD_ROLLZONE);	};
 sub ROLLCMD_ROLLZSK		{ return($ROLLCMD_ROLLZSK);	};
 sub ROLLCMD_RUNQUEUE		{ return($ROLLCMD_RUNQUEUE);	};
 sub ROLLCMD_SHUTDOWN		{ return($ROLLCMD_SHUTDOWN);	};
@@ -275,10 +292,14 @@ my %roll_commands =
 	rollcmd_logfile		=> 1,
 	rollcmd_loglevel	=> 1,
 	rollcmd_logmsg		=> 1,
+	rollcmd_logtz		=> 1,
 	rollcmd_nodisplay	=> 1,
+	rollcmd_phasemsg	=> 1,
+	rollcmd_rollall		=> 1,
 	rollcmd_rollallzsks	=> 1,
 	rollcmd_rollksk		=> 1,
 	rollcmd_rollrec		=> 1,
+	rollcmd_rollzone	=> 1,
 	rollcmd_rollzsk		=> 1,
 	rollcmd_runqueue	=> 1,
 	rollcmd_shutdown	=> 1,
@@ -447,7 +468,8 @@ my %key_phases =
 #
 my $PHASELONG	= 1;				# Use long descriptions.
 my $PHASESHORT	= 0;				# Use short descriptions.
-my $longorshort = $PHASELONG;			# Description length to use.
+# my $longorshort = $PHASELONG;			# Description length to use.
+my $longorshort = $PHASESHORT;			# Description length to use.
 
 ##############################################################################
 ##############################################################################
@@ -460,7 +482,6 @@ my $longorshort = $PHASELONG;			# Description length to use.
 #
 
 #--------------------------------------------------------------------------
-#
 # Routine:      rollmgr_prepdep()
 #
 # Purpose:	This routine prepares for device-dependent calls.  A global
@@ -502,12 +523,11 @@ sub rollmgr_prepdep
 	# Get the appropriate switch table for this O/S class and save
 	# it for later reference.
 	#
-	my $swtab = $port_archs{$osclass};
+	$swtab = $port_archs{$osclass};
 	%switchtab = %$swtab;
 }
 
 #--------------------------------------------------------------------------
-#
 # Routine:      rollmgr_dir()
 #
 # Purpose:	Front-end to the O/S-specific "get rollerd's
@@ -525,7 +545,6 @@ sub rollmgr_dir
 }
 
 #--------------------------------------------------------------------------
-#
 # Routine:      rollmgr_dropid()
 #
 # Purpose:	Front-end to the O/S-specific "save rollerd's
@@ -543,7 +562,6 @@ sub rollmgr_dropid
 }
 
 #--------------------------------------------------------------------------
-#
 # Routine:      rollmgr_getid()
 #
 # Purpose:	Front-end to the O/S-specific "get rollerd's
@@ -561,7 +579,6 @@ sub rollmgr_getid
 }
 
 #--------------------------------------------------------------------------
-#
 # Routine:      rollmgr_halt()
 #
 # Purpose:	Front-end to the O/S-specific "halt rollerd"
@@ -579,7 +596,6 @@ sub rollmgr_halt
 }
 
 #--------------------------------------------------------------------------
-#
 # Routine:      rollmgr_idfile()
 #
 # Purpose:	Front-end to the O/S-specific "get rollerd's
@@ -597,7 +613,6 @@ sub rollmgr_idfile
 }
 
 #--------------------------------------------------------------------------
-#
 # Routine:      rollmgr_set_idfile()
 #
 # Purpose:	Front-end to the O/S-specific "set rollerd's
@@ -615,7 +630,6 @@ sub rollmgr_set_idfile
 }
 
 #--------------------------------------------------------------------------
-#
 # Routine:      rollmgr_loadzone()
 #
 # Purpose:	Front-end to the O/S-specific "load the zone" function.
@@ -632,7 +646,33 @@ sub rollmgr_loadzone
 }
 
 #--------------------------------------------------------------------------
+# Routine:      rollmgr_phasemsg()
 #
+# Purpose:	Set the phase-message length.  Valid values are "long" 
+#		and "short".
+#		This is generic and not O/S-specific.
+#
+sub rollmgr_phasemsg
+{
+	my $pval = shift;			# The new message length.
+
+# print "rollmgr_phasemsg\n";
+
+	if($pval =~ /long/i)
+	{
+		$longorshort = $PHASELONG; 
+		return(1);
+	}
+	elsif($pval =~ /short/i)
+	{
+		$longorshort = $PHASESHORT; 
+		return(1);
+	}
+
+	return(0);
+}
+
+#--------------------------------------------------------------------------
 # Routine:      rollmgr_cmdint()
 #
 # Purpose:	Front-end to the O/S-specific "rollerd has a
@@ -650,7 +690,6 @@ sub rollmgr_cmdint
 }
 
 #--------------------------------------------------------------------------
-#
 # Routine:      rollmgr_rmid()
 #
 # Purpose:	Front-end to the O/S-specific "remove rollerd's
@@ -668,7 +707,6 @@ sub rollmgr_rmid
 }
 
 #--------------------------------------------------------------------------
-#
 # Routine:      rollmgr_running()
 #
 # Purpose:	Front-end to the O/S-specific "is rollerd running?" function.
@@ -685,7 +723,6 @@ sub rollmgr_running
 }
 
 #--------------------------------------------------------------------------
-#
 # Routine:      rollmgr_saveid()
 #
 # Purpose:	Front-end to the O/S-specific "save rollerd's
@@ -717,7 +754,6 @@ sub rollmgr_saveid
 #	call the proper O/S-dependent interface.
 
 #--------------------------------------------------------------------------
-#
 # Routine:      uninit_dir()
 #
 # Purpose:	Switch for uninitialized "get dir" command.
@@ -733,7 +769,6 @@ sub uninit_dir
 }
 
 #--------------------------------------------------------------------------
-#
 # Routine:      uninit_dropid()
 #
 # Purpose:	Switch for uninitialized "drop id" command.
@@ -749,7 +784,6 @@ sub uninit_dropid
 }
 
 #--------------------------------------------------------------------------
-#
 # Routine:      uninit_getid()
 #
 # Purpose:	Switch for uninitialized "get id" command.
@@ -765,7 +799,6 @@ sub uninit_getid
 }
 
 #--------------------------------------------------------------------------
-#
 # Routine:      uninit_halt()
 #
 # Purpose:	Switch for uninitialized "halt" command.
@@ -781,7 +814,6 @@ sub uninit_halt
 }
 
 #--------------------------------------------------------------------------
-#
 # Routine:      uninit_idfile()
 #
 # Purpose:	Switch for uninitialized "get id file" command.
@@ -797,7 +829,6 @@ sub uninit_idfile
 }
 
 #--------------------------------------------------------------------------
-#
 # Routine:      uninit_idfile()
 #
 # Purpose:	Switch for uninitialized "get id file" command.
@@ -813,7 +844,6 @@ sub uninit_set_idfile
 }
 
 #--------------------------------------------------------------------------
-#
 # Routine:      uninit_loadzone()
 #
 # Purpose:	Switch for uninitialized "load the zone" command.
@@ -829,7 +859,6 @@ sub uninit_loadzone
 }
 
 #--------------------------------------------------------------------------
-#
 # Routine:      uninit_cmdint()
 #
 # Purpose:	Switch for uninitialized "force queue" command.
@@ -845,7 +874,6 @@ sub uninit_cmdint
 }
 
 #--------------------------------------------------------------------------
-#
 # Routine:      uninit_rmid()
 #
 # Purpose:	Switch for uninitialized "remove id file" command.
@@ -861,7 +889,6 @@ sub uninit_rmid
 }
 
 #--------------------------------------------------------------------------
-#
 # Routine:      uninit_running()
 #
 # Purpose:	Switch for uninitialized "is rollerd running?" command.
@@ -877,7 +904,6 @@ sub uninit_running
 }
 
 #--------------------------------------------------------------------------
-#
 # Routine:      uninit_saveid()
 #
 # Purpose:	Switch for uninitialized "save id file" command.
@@ -905,7 +931,6 @@ sub uninit_saveid
 #
 
 #--------------------------------------------------------------------------
-#
 # Routine:      unknown_action()
 #
 sub unknown_action
@@ -914,7 +939,6 @@ sub unknown_action
 }
 
 #--------------------------------------------------------------------------
-#
 # Routine:      unknown_dir()
 #
 sub unknown_dir
@@ -923,7 +947,6 @@ sub unknown_dir
 }
 
 #--------------------------------------------------------------------------
-#
 # Routine:      unknown_getid()
 #
 sub unknown_getid
@@ -932,7 +955,6 @@ sub unknown_getid
 }
 
 #--------------------------------------------------------------------------
-#
 # Routine:      unknown_halt()
 #
 sub unknown_halt
@@ -941,7 +963,6 @@ sub unknown_halt
 }
 
 #--------------------------------------------------------------------------
-#
 # Routine:      unknown_idfile()
 #
 sub unknown_idfile
@@ -950,7 +971,6 @@ sub unknown_idfile
 }
 
 #--------------------------------------------------------------------------
-#
 # Routine:      unknown_set_idfile()
 #
 sub unknown_set_idfile
@@ -959,7 +979,6 @@ sub unknown_set_idfile
 }
 
 #--------------------------------------------------------------------------
-#
 # Routine:      unknown_loadzone()
 #
 sub unknown_loadzone
@@ -968,7 +987,6 @@ sub unknown_loadzone
 }
 
 #--------------------------------------------------------------------------
-#
 # Routine:      unknown_cmdint()
 #
 sub unknown_cmdint
@@ -977,7 +995,6 @@ sub unknown_cmdint
 }
 
 #--------------------------------------------------------------------------
-#
 # Routine:      unknown_rmid()
 #
 sub unknown_rmid
@@ -986,7 +1003,6 @@ sub unknown_rmid
 }
 
 #--------------------------------------------------------------------------
-#
 # Routine:      unknown_running()
 #
 sub unknown_running
@@ -995,7 +1011,6 @@ sub unknown_running
 }
 
 #--------------------------------------------------------------------------
-#
 # Routine:      unknown_saveid()
 #
 sub unknown_saveid
@@ -1015,7 +1030,6 @@ sub unknown_saveid
 #
 
 #--------------------------------------------------------------------------
-#
 # Routine:      unix_dir()
 #
 sub unix_dir
@@ -1024,7 +1038,6 @@ sub unix_dir
 }
 
 #--------------------------------------------------------------------------
-#
 # Routine:	unix_idfile()
 #
 # Purpose:	Return rollerd's id file.
@@ -1035,7 +1048,6 @@ sub unix_idfile
 }
 
 #--------------------------------------------------------------------------
-#
 # Routine:	unix_set_idfile()
 #
 # Purpose:	Sets rollerd's id file to a particular value
@@ -1046,7 +1058,6 @@ sub unix_set_idfile
 }
 
 #--------------------------------------------------------------------------
-#
 # Routine:	unix_loadzone()
 #
 # Purpose:	Kick the name server so it'll load the given zone.
@@ -1075,7 +1086,6 @@ sub unix_loadzone
 }
 
 #--------------------------------------------------------------------------
-#
 # Routine:	unix_psef_dropid()
 #
 # Purpose:      Replaces unix_dropid on ps -ef based systems
@@ -1086,7 +1096,6 @@ sub unix_psef_dropid
 }
 
 #--------------------------------------------------------------------------
-#
 # Routine:	unix_dropid()
 #
 # Purpose:	Ensures that another instance of rollerd is not running and
@@ -1219,7 +1228,6 @@ sub unix_dropid
 
 
 #--------------------------------------------------------------------------
-#
 # Routine:	unix_rmid()
 #
 # Purpose:	Delete rollerd's pidfile.  This is done as part of the
@@ -1281,7 +1289,6 @@ sub unix_rmid
 }
 
 #--------------------------------------------------------------------------
-#
 # Routine:	unix_running()
 #
 # Purpose:	Determine if rollerd is running and return a boolean
@@ -1329,7 +1336,6 @@ sub unix_running
 }
 
 #--------------------------------------------------------------------------
-#
 # Routine:	unix_getpid()
 #
 # Purpose:	Return rollerd, as recorded in its pidfile.
@@ -1393,7 +1399,6 @@ sub unix_getpid
 }
 
 #--------------------------------------------------------------------------
-#
 # Routine:	unix_cmdint()
 #
 # Purpose:	Kick rollerd to let it know it should re-read the rollrec
@@ -1421,7 +1426,6 @@ sub unix_cmdint
 }
 
 #--------------------------------------------------------------------------
-#
 # Routine:	unix_halt()
 #
 # Purpose:	Tell rollerd to shut down.
@@ -1452,7 +1456,6 @@ sub unix_halt
 #############################################################################
 
 #-----------------------------------------------------------------------------
-#
 # Routine:	rollmgr_channel()
 #
 # Purpose:	This routine initializes a socket to use for rollerd
@@ -1554,7 +1557,6 @@ sub rollmgr_channel
 }
 
 #-----------------------------------------------------------------------------
-#
 # Routine:	rollmgr_closechan()
 #
 # Purpose:	This routine closes down the communications channel to
@@ -1567,15 +1569,14 @@ sub rollmgr_closechan
 }
 
 #-----------------------------------------------------------------------------
-#
 # Routine:	rollmgr_queuecmd()
 #
 # Purpose:	This routine can be called internally to queue a command
 #		for later processing via calls to rollmgr_getcmd().
-#		It is useful when doing initial startup before full
+#		It is useful when doing initial start-up before full
 #		processing is to commence.  Commands queued by this
 #		process take precedence over commands received via the
-#		command intervace (ie, via rollmgr_sendcmd()).
+#		command interface (ie, via rollmgr_sendcmd()).
 #
 sub rollmgr_queuecmd
 {
@@ -1586,7 +1587,6 @@ sub rollmgr_queuecmd
 }
 
 #-----------------------------------------------------------------------------
-#
 # Routine:	rollmgr_getqueueitem()
 #
 # Purpose:	This routine can be called pull a command from the queue
@@ -1600,16 +1600,13 @@ sub rollmgr_getqueueitem
 		my $cmd = shift @queuedcmds;
 		return($cmd);
 	}
-	return;
 }
 
-
 #-----------------------------------------------------------------------------
-#
 # Routine:	rollmgr_getallqueuedcmds()
 #
 # Purpose:	This routine returns all the queued commands in the stack
-#		The items are left in place unless an truthful argument
+#		The items are left in place unless a truthful argument
 #		(e.g. "1") is passed in.
 #
 sub rollmgr_getallqueuedcmds
@@ -1621,10 +1618,7 @@ sub rollmgr_getallqueuedcmds
 	return(@queuedcmds);
 }
 
-
-
 #-----------------------------------------------------------------------------
-#
 # Routine:	rollmgr_getcmd()
 #
 # Purpose:	This routine is called by the server to fetch a command and
@@ -1714,7 +1708,6 @@ sub rollmgr_getcmd
 }
 
 #-----------------------------------------------------------------------------
-#
 # Routine:	rollmgr_sendcmd()
 #
 # Purpose:	This routine allows a client to send a message to the server.
@@ -1770,7 +1763,6 @@ sub rollmgr_sendcmd
 }
 
 #-----------------------------------------------------------------------------
-#
 # Routine:	rollmgr_sendresp()
 #
 # Purpose:	This routine allows rollerd to send a message to a client.
@@ -1803,7 +1795,6 @@ sub rollmgr_sendresp
 }
 
 #-----------------------------------------------------------------------------
-#
 # Routine:	rollmgr_getresp()
 #
 # Purpose:	This routine allows a client to wait for a message response
@@ -1848,7 +1839,6 @@ sub rollmgr_getresp
 }
 
 #-----------------------------------------------------------------------------
-#
 # Routine:	rollmgr_verifycmd()
 #
 # Purpose:	This routine returns a boolean indicating if the specified
@@ -1869,7 +1859,6 @@ sub rollmgr_verifycmd
 }
 
 #-----------------------------------------------------------------------------
-#
 # Routine:	rollmgr_get_phase()
 #
 # Purpose:	This routine translates the numerical phases of rolling
@@ -1882,7 +1871,7 @@ sub rollmgr_get_phase
 	my $maxphase = -1;				# Maximum phase value.
 
 	#
-	# Don't return anything if the installer doesn't want long descritions.
+	# Don't return anything if the installer doesn't want long descriptions.
 	#	NOTE:  This really should be a config value.
 	#
 	if($longorshort != $PHASELONG)
@@ -1941,6 +1930,8 @@ manager.
 
   rollmgr_halt();
 
+  rollmgr_phasemsg('long');
+
   rollmgr_channel(1);
   ($cmd,$data) = rollmgr_getcmd();
   $ret = rollmgr_verifycmd($cmd);
@@ -2043,6 +2034,26 @@ returned.
     signaled.
     (This should only ever be 1.)
 
+=item I<rollmgr_phasemsg()>
+
+This routine sets the phase-message length.  of phase-related log messages
+used by B<rollerd>.  The valid levels are "long" and "short", with "long"
+being the default value.
+
+The long message length means that a phase description will be included with
+some log messages.  For example, the long form of a message about ZSK rollover
+phase 3 will look like this:  "ZSK phase 3 (Waiting for old zone data to
+expire from caches)".
+
+The short message length means that a phase description will not be included
+with some log messages.  For example, the short form of a message about ZSK
+rollover phase 3 will look like this:  "ZSK phase 3".
+
+Return Values:
+
+     1 - the phase-message length was set
+     0 - an invalid phase-message length was specified
+
 =back
 
 =head1 ROLLERD COMMUNICATIONS INTERFACES
@@ -2096,18 +2107,26 @@ The available commands and their required data are:
    ROLLCMD_LOGFILE	log filename	change the log file
    ROLLCMD_LOGLEVEL	log level	set a new logging level
    ROLLCMD_LOGMSG	log message	add a message to the log
-   ROLLCMD_ROLLALL	none		force all zones to start
+   ROLLCMD_LOGTZ	timezone	set timezone for log messages
+   ROLLCMD_PHASEMSG	long/short	set long or short phase
+					messages
+   ROLLCMD_ROLLALL	none		resume rollover for all
+					suspended zones
+   ROLLCMD_ROLLALLZSKS	none		force all zones to start
 					ZSK rollover
    ROLLCMD_ROLLKSK	zone-name	force a zone to start
 					KSK rollover
    ROLLCMD_ROLLREC	rollrec-name	change rollerd's rollrec file
+   ROLLCMD_ROLLZONE	zone name	restart rollover for a
+					suspended zone
    ROLLCMD_ROLLZSK	zone-name	force a zone to start
 					ZSK rollover
    ROLLCMD_RUNQUEUE	none		rollerd runs through
 					its queue
    ROLLCMD_SHUTDOWN	none		stop rollerd
    ROLLCMD_SKIPALL	none		suspend all rollovers
-   ROLLCMD_SKIPZONE	zone name	suspend rollover for a zone
+   ROLLCMD_SKIPZONE	zone name	suspend rollover for a
+					rolling zone
    ROLLCMD_SLEEPTIME	seconds-count	set rollerd's sleep time
    ROLLCMD_STATUS	none		get status of rollerd
    ROLLCMD_ZONELOG	zone name	set the logging level for
@@ -2164,7 +2183,7 @@ the file prior to opening it, but we can't do so without it being open.
 
 =head1 COPYRIGHT
 
-Copyright 2005-2010 SPARTA, Inc.  All rights reserved.
+Copyright 2005-2011 SPARTA, Inc.  All rights reserved.
 See the COPYING file included with the DNSSEC-Tools package for details.
 
 =head1 AUTHOR
