@@ -1,6 +1,6 @@
 #!/usr/bin/perl
 #
-# Copyright 2005-2010 SPARTA, Inc.  All rights reserved.  See the COPYING
+# Copyright 2005-2011 SPARTA, Inc.  All rights reserved.  See the COPYING
 # file distributed with this software for details.
 #
 # DNSSEC-Tools
@@ -27,7 +27,8 @@ use strict;
 use Net::DNS::SEC::Tools::conf;
 use Net::DNS::SEC::Tools::keyrec;
 
-our $VERSION = "1.8";
+our $VERSION = "1.9";
+our $MODULE_VERSION = "1.9.0";
 
 our @ISA = qw(Exporter);
 
@@ -121,6 +122,7 @@ my @stdopts =
 	['GUI:separator',	'Roll-over Options:'],
 		["roll_logfile=s",	"Roll-over manager's log filename"],
 		["roll_loglevel=s",	"Roll-over manager's logging level"],
+		["logtz=s",		"Roll-over manager's logging timezone"],
 		["roll_sleeptime=i",	"Sleep-time for Roll-over manager"],
 		["zone_errors=i",	"Maximum consecutive errors for a zone"],
 
@@ -1045,7 +1047,7 @@ The old exit action is returned.
 
 =head1 COPYRIGHT
 
-Copyright 2005-2010 SPARTA, Inc.  All rights reserved.
+Copyright 2005-2011 SPARTA, Inc.  All rights reserved.
 See the COPYING file included with the DNSSEC-Tools package for details.
 
 =head1 AUTHOR
