@@ -1,5 +1,5 @@
 /*
- * Copyright 2005-2009 SPARTA, Inc.  All rights reserved.
+ * Copyright 2005-2011 SPARTA, Inc.  All rights reserved.
  * See the COPYING file distributed with this software for details.
  *
  * Author: Abhijit Hayatnagarkar
@@ -27,6 +27,10 @@
 #include <validator/resolver.h>
 #include <validator/validator.h>
 
+#define	NAME	"getaddr"
+#define	VERS	"version: 1.0"
+#define	DTVERS	"DNSSEC-Tools version: 1.8"
+
 #ifdef HAVE_GETOPT_LONG
 // Program options
 static struct option prog_options[] = {
@@ -34,6 +38,7 @@ static struct option prog_options[] = {
     {"novalidate", 0, 0, 'n'},
     {"canonname", 0, 0, 'c'},
     {"service", 0, 0, 's'},
+    {"Version", 0, 0, 'V'},
     {0, 0, 0, 0}
 };
 #endif
@@ -61,6 +66,15 @@ usage(char *progname)
             "\t              file:<file-name>   (opened in append mode)\n" 
             "\t              net[:<host-name>:<host-port>] (127.0.0.1:1053\n" 
             "\t              syslog[:facility] (0-23 (default 1 USER))\n" );
+    fprintf(stderr,
+            "\t-V, --Version                   display version and exit\n");
+}
+
+void
+version(void)
+{
+    fprintf(stderr, "%s %s\n", NAME, VERS);
+    fprintf(stderr, "%s\n", DTVERS);
 }
 
 static int      validate = 0;
@@ -167,13 +181,13 @@ main(int argc, char *argv[])
 #ifdef HAVE_GETOPT_LONG
         int             opt_index = 0;
 #ifdef HAVE_GETOPT_LONG_ONLY
-        c = getopt_long_only(argc, argv, "hcno:s:",
+        c = getopt_long_only(argc, argv, "hcno:s:V",
                              prog_options, &opt_index);
 #else
-        c = getopt_long(argc, argv, "hcno:s:", prog_options, &opt_index);
+        c = getopt_long(argc, argv, "hcno:s:V", prog_options, &opt_index);
 #endif
 #else                           /* only have getopt */
-        c = getopt(argc, argv, "hcno:s:");
+        c = getopt(argc, argv, "hcno:s:V");
 #endif
 
         if (c == -1) {
@@ -202,6 +216,9 @@ main(int argc, char *argv[])
         case 'c':
             getcanonname = 1;
             break;
+        case 'V':
+            version();
+            return 0;
         default:
             fprintf(stderr, "Invalid option %s\n", argv[optind - 1]);
             usage(argv[0]);
