@@ -1,5 +1,5 @@
 /*
- * Copyright 2005-2009 SPARTA, Inc.  All rights reserved.
+ * Copyright 2005-2011 SPARTA, Inc.  All rights reserved.
  * See the COPYING file distributed with this software for details.
  */
 
@@ -41,6 +41,11 @@
 #include <validator/validator.h>
 #include "validator_driver.h"
 
+#define	NAME	"validate"
+#define	VERS	"version: 1.0"
+#define	DTVERS	 "DNSSEC-Tools Version: 1.8"
+
+
 #define NO_OF_THREADS 0
 
 #if NO_OF_THREADS
@@ -74,6 +79,8 @@ static struct option prog_options[] = {
     {"dnsval-conf", 1, 0, 'v'},
     {"root-hints", 1, 0, 'i'},
     {"wait", 1, 0, 'w'},
+    {"inflight", 1, 0, 'I'},
+    {"Version", 1, 0, 'V'},
     {0, 0, 0, 0}
 };
 #endif
@@ -245,7 +252,7 @@ usage(char *progname)
     printf("Primary Options:\n");
     printf("        -h, --help             Display this help and exit\n");
     printf("        -p, --print            Print the answer and validation result\n");
-    printf("        -s, --selftest         Run all internal sefltest suite(s)\n");
+    printf("        -s, --selftest         Run all internal selftest suite(s)\n");
     printf("        -S, --test-suite=<suite>[:<suite>] Run specified internal sefltest suite(s)\n");
     printf("        -T, --testcase=<number>[:<number>\n");
     printf("                               Specifies the test case number/range \n");
@@ -255,6 +262,7 @@ usage(char *progname)
     printf("        -v, --dnsval-conf=<file> Specifies a dnsval.conf\n");
     printf("        -r, --resolv-conf=<file> Specifies a resolv.conf to search for nameservers\n");
     printf("        -i, --root-hints=<file> Specifies a root.hints to search for root nameservers\n");
+    printf("        -I, --inflight=<number> Maximum number of simultaneous queries\n");
     printf("        -w, --wait=<secs> Run tests in a loop, sleeping for specifed seconds between runs\n");
     printf("        -l, --label=<label-string> Specifies the policy to use during validation\n");
     printf("        -o, --output=<debug-level>:<dest-type>[:<dest-options>]\n");
@@ -264,6 +272,7 @@ usage(char *progname)
     printf("                  file:<file-name>   (opened in append mode)\n");
     printf("                  net[:<host-name>:<host-port>] (127.0.0.1:1053\n");
     printf("                  syslog[:facility] (0-23 (default 1 USER))\n");
+    printf("        -V, --Version          Display version and exit\n");
     printf("Advanced Options:\n");
     printf("\nThe DOMAIN_NAME parameter is not required for the -h option.\n");
     printf("The DOMAIN_NAME parameter is required if one of -p, -c or -t options is given.\n");
@@ -271,6 +280,13 @@ usage(char *progname)
     /* *INDENT-ON* */
 }
 
+void
+version(void)
+{
+    fprintf(stderr, "%s: %s\n",NAME,VERS);
+    fprintf(stderr, "%s\n",DTVERS);
+}
+
 /*============================================================================
  *
  * DAEMON MODE SUPPORT FUNCTIONS BEGIN HERE
@@ -450,8 +466,8 @@ process_packet(val_context_t *context)
 
     response_size = sizeof(response);
     
-    get_results(context, "test", &query[sizeof(HEADER)], (int)q_class, (int)q_type,
-                response, &response_size, 0);
+    get_results(context, "test", (char *)&query[sizeof(HEADER)], (int)q_class,
+                (int)q_type, response, &response_size, 0);
 
     /*
      * check to see if we need a dummy response
@@ -522,6 +538,7 @@ one_test(val_context_t *context, char *name, int class_h,
         int type_h, u_int32_t flags, int retvals[], int doprint)
 {
     struct val_response resp;
+    memset(&resp, 0, sizeof(struct val_response));
     sendquery(context, "Result", name, class_h, type_h, flags, retvals, 1, &resp);
     fprintf(stderr, "\n");
 
@@ -545,6 +562,7 @@ struct thread_params_st {
     char *suite;
     int doprint;
     int wait;
+    int max_in_flight;
 };
 
 struct thread_params_ot {
@@ -566,7 +584,7 @@ void *firethread_st(void *param) {
             (unsigned int)threadparams->context);
     do {
         self_test(threadparams->context, threadparams->tcs, threadparams->tce, threadparams->flags, 
-                  threadparams->testcase_config, threadparams->suite, threadparams->doprint);
+                  threadparams->testcase_config, threadparams->suite, threadparams->doprint, threadparams->max_in_flight);
         if (threadparams->wait)
             sleep(threadparams->wait);
     }while (threadparams->wait);
@@ -613,12 +631,13 @@ main(int argc, char *argv[])
     // Parse the command line for a query and resolve+validate it
     int             c;
     char           *domain_name = NULL;
-    const char     *args = "c:dF:hi:l:w:o:pr:S:st:T:v:";
+    const char     *args = "c:dF:hi:I:l:w:o:pr:S:st:T:v:V";
     int            class_h = ns_c_in;
     int            type_h = ns_t_a;
     int             success = 0;
     int             doprint = 0;
     int             selftest = 0;
+    int             max_in_flight = 1;
     int             daemon = 0;
     u_int32_t       flags = 0;
     int             retvals[] = { 0 };
@@ -713,6 +732,10 @@ main(int argc, char *argv[])
             break;
 
 
+        case 'I':
+            max_in_flight = strtol(optarg, &nextarg, 10);
+            break;
+
         case 'v':
             dnsval_conf_set(optarg);
             break;
@@ -750,6 +773,10 @@ main(int argc, char *argv[])
             label_str = optarg;
             break;
 
+        case 'V':
+            version();
+            return 0;
+
         default:
             fprintf(stderr, "Unknown option %s (c = %d [%c])\n",
                     argv[optind - 1], c, (char) c);
@@ -774,6 +801,10 @@ main(int argc, char *argv[])
     context = NULL;
 #endif
 
+    /* returned level is 0 based;, > 6 means 8 or higher; e.g. -o 8:stdout */
+    if (val_log_highest_debug_level() > 6)
+        res_io_set_debug(1);
+
     rc = 0;
 
     // optind is a global variable.  See man page for getopt_long(3)
@@ -802,7 +833,7 @@ main(int argc, char *argv[])
 #else
             do { /* endless loop */ 
                 rc = self_test(context, tcs, tce, flags, testcase_config, suite,
-                               doprint);
+                               doprint, max_in_flight);
                 if (wait)
                     sleep(wait);
             } while (wait && !rc);
