@@ -1,5 +1,5 @@
 /*
- * Copyright 2007-2009 SPARTA, Inc.  All rights reserved.
+ * Copyright 2007-2011 SPARTA, Inc.  All rights reserved.
  * See the COPYING file distributed with this software for details.
  */
 #ifndef VALIDATOR_DRIVER_H
@@ -12,7 +12,8 @@ int sendquery(val_context_t * context, const char *desc, char * name,
               struct val_response *resp);
 
 int self_test(val_context_t *context, int tcs, int tce, u_int32_t flags,
-              const char *tests, const char *suites, int doprint);
+              const char *tests, const char *suites, int doprint,
+              int max_in_flight);
 
 
 
