@@ -16,7 +16,7 @@
  * WITH THE USE OR PERFORMANCE OF THE SOFTWARE.
  */
 /*
- * Copyright 2005-2009 SPARTA, Inc.  All rights reserved.
+ * Copyright 2005-2011 SPARTA, Inc.  All rights reserved.
  * See the COPYING file distributed with this software for details.
  */
 #ifndef RESOLVER_H
@@ -41,6 +41,7 @@ extern          "C" {
 #define STRDUP(p) strdup(p)
 #endif
 
+#define IPADDR_STRING_MAX 128
 
 #define RES_RETRY 1 /* number of times to retry */
 #define LIBSRES_NS_STAGGER 5 /* how far apart should we stagger queries to
@@ -50,7 +51,11 @@ extern          "C" {
 #endif
 
 #ifndef RES_USE_DNSSEC
-#define RES_USE_DNSSEC  0x00200000
+#define RES_USE_DNSSEC  0x00800000
+#endif
+
+#ifndef RES_USE_EDNS0
+#define RES_USE_EDNS0   0x00100000
 #endif
 
 #define RES_EDNS0_DEFAULT 4096
@@ -223,6 +228,10 @@ int             get(const char *name_n,
                     struct name_server **server,
                     u_char ** response, size_t * response_length);
 void            print_response(u_char * ans, size_t resplen);
+
+struct sockaddr_storage **create_nsaddr_array(int num_addrs);
+struct name_server *parse_name_server(const char *cp,
+                                      const char *name_n);
 int             clone_ns(struct name_server **cloned_ns,
                          struct name_server *ns);
 int             clone_ns_list(struct name_server **ns_list,
@@ -230,6 +239,9 @@ int             clone_ns_list(struct name_server **ns_list,
 void            free_name_server(struct name_server **ns);
 void            free_name_servers(struct name_server **ns);
 
+void            res_io_set_debug(int val);
+int             res_io_get_debug(void);
+
 #ifndef HAVE_DECL_NS_NTOP
 int             ns_name_ntop(const u_char * src, char *dst, size_t dstsiz);
 #endif
@@ -262,6 +274,47 @@ const char     *p_class(int pclass);
 #define p_type(type) p_sres_type(type)
 const char     *p_sres_type(int type);
 
+/*
+ * asynchronous interface to the resolver
+ */
+
+/** foward declare of opaque structure used by async routines */
+struct expected_arrival;
+
+struct expected_arrival *
+res_async_query_send(const char *name, const u_int16_t type_h,
+                     const u_int16_t class_h, struct name_server *pref_ns);
+
+void
+res_async_query_select_info(struct expected_arrival *ea, int *nfds,
+                            fd_set *fds, struct timeval *timeout);
+int
+res_async_query_handle(struct expected_arrival *ea, int *handled, fd_set *fds);
+
+void
+res_async_query_free(struct expected_arrival *ea);
+
+void
+res_io_cancel_remaining_attempts(struct expected_arrival *ea);
+
+void
+res_io_cancel_all_remaining_attempts(struct expected_arrival *ea);
+
+int
+res_io_is_finished(struct expected_arrival *ea);
+
+int
+res_io_are_all_finished(struct expected_arrival *ea);
+
+int
+res_async_ea_is_using_stream(struct expected_arrival *ea);
+
+int
+res_async_ea_isset(struct expected_arrival *ea, fd_set *fds);
+
+int
+libsres_msg_getflag(ns_msg han, int flag);
+
 /*
  * at one open ns_msg_getflag was a macro on Linux, but now it is a
  * function in libresolv. redifine to use our internal version.
@@ -270,7 +323,6 @@ const char     *p_sres_type(int type);
 #define ns_msg_getflag libsres_msg_getflag
 #endif
 
-
 #ifdef __cplusplus
 }                               /* extern C */
 #endif
