@@ -1,6 +1,6 @@
 
 /*
- * Copyright 2005-2009 SPARTA, Inc.  All rights reserved.
+ * Copyright 2005-2011 SPARTA, Inc.  All rights reserved.
  * See the COPYING file distributed with this software for details.
  */
 #ifndef VALIDATOR_H
@@ -195,25 +195,20 @@ extern          "C" {
 #define VAL_FROM_ADDITIONAL       3
 #define VAL_FROM_QUERY            4
 
-    /*
-     * user query flags in the lower two bytes 
-     */
-#define VAL_QFLAGS_ANY 0xffffffff
-#define VAL_QFLAGS_USERMASK 0x0000ffff
-#define VAL_QFLAGS_STATE 0xffff0000
-#define VAL_QFLAGS_CACHE_MASK   VAL_QFLAGS_USERMASK
+#define VAL_QFLAGS_ANY          0xffffffff
+#define VAL_QFLAGS_CACHE_MASK   0x00ffffff
+#define VAL_QFLAGS_USERMASK     0x0000ffff
 
-#define VAL_QUERY_NO_AC_DETAIL 0x00000001
-#define VAL_QUERY_NO_EDNS0 0x00000002
-#define VAL_QUERY_DONT_VALIDATE 0x00000004 
+#define VAL_QUERY_NO_AC_DETAIL  0x00000001
+#define VAL_QUERY_DONT_VALIDATE 0x00000002 
+#define VAL_QUERY_NO_DLV        0x00000004 
+#define VAL_QUERY_REFRESH_QCACHE 0x00000008
+#define VAL_QUERY_ASYNC         0x00000010
 
-    /*  
-     * Internal query state in the upper two bytes 
-     */
-#define VAL_QUERY_GLUE_REQUEST (0x00010000 | VAL_QUERY_DONT_VALIDATE)
-#define VAL_QUERY_USING_DLV 0x00020000 
-#define VAL_QUERY_NO_DLV 0x00040000 
+#define VAL_QUERY_NO_EDNS0      0x00010000
+#define VAL_QUERY_USING_DLV     0x00020000 
 
+#define VAL_QUERY_GLUE_REQUEST (0x01000000 | VAL_QUERY_DONT_VALIDATE)
 
 #define MAX_ALIAS_CHAIN_LENGTH 10       /* max length of cname/dname chain */
 #define MAX_GLUE_FETCH_DEPTH 10         /* max length of glue dependency chain */
@@ -238,6 +233,7 @@ extern          "C" {
         int env_policy;
         int app_policy;
         char *log_target;
+        int closest_ta_only;
     } global_opt_t;
 
     typedef struct {
@@ -270,14 +266,15 @@ extern          "C" {
 
 #define GOPT_TRUST_OOB_STR "trust-oob-answers"
 #define GOPT_EDNS0_SIZE_STR "edns0-size"
-#define TRUST_OOB_GOPT_YES_STR "yes"
-#define TRUST_OOB_GOPT_NO_STR "no"
+#define GOPT_YES_STR "yes"
+#define GOPT_NO_STR "no"
 #define GOPT_ENV_POL_STR "env-policy"
 #define GOPT_APP_POL_STR "app-policy"
 #define GOPT_ENABLE_STR "enable"
 #define GOPT_DISBLE_STR "disable"
 #define GOPT_OVERRIDE_STR "override"
 #define GOPT_LOGTARGET_STR "log"
+#define GOPT_CLOSEST_TA_ONLY_STR "closest-ta-only"
 
 #define VAL_POL_GOPT_DISABLE 0 
 #define VAL_POL_GOPT_ENABLE 1
@@ -432,6 +429,43 @@ extern          "C" {
 
 #endif
 
+#ifndef VAL_NO_ASYNC
+    /*
+     * asynchronous status
+     */
+#define VAL_AS_CTX_USER_SUPPLIED     0x00000001 /* i.e. don't delete it! */
+#define VAL_AS_IGNORE_CACHE          0x00000002
+#define VAL_AS_NO_NEW_QUERIES        0x00000004
+#define VAL_AS_DONE                  0x00000008 /* have results/answers */
+#define VAL_AS_CB_COMPLETED          0x00000010 /* called user callbacks */
+#define VAL_AS_NO_ANSWERS            0x00000020 /* don't care about answers */
+#define VAL_AS_NO_CALLBACKS          0x00000040 /* don't call callbacks */
+
+    typedef struct val_async_status_s val_async_status;
+    typedef int (*val_cb_results)(val_async_status *as);
+
+    struct val_async_status_s {
+        val_context_t                 *val_as_ctx;
+        u_int32_t                      val_as_flags;
+
+        u_char                         val_as_inflight;
+        struct queries_for_query      *val_as_top_q;
+        struct queries_for_query      *val_as_queries;
+
+        u_char                         val_as_domain_name_n[NS_MAXCDNAME];
+        int                            val_as_class;
+        int                            val_as_type;
+
+        struct val_result_chain       *val_as_results;
+        struct val_answer_chain       *val_as_answers;
+
+        val_cb_results                 val_as_result_cb;
+        void                          *val_as_cb_user_ctx;
+
+        struct val_async_status_s     *val_as_next;
+    };
+#endif /* VAL_NO_ASYNC */
+
     /*
      * Logging-related definitions 
      */
@@ -500,6 +534,7 @@ extern          "C" {
 
     int             val_log_debug_level(void);
     void            val_log_set_debug_level(int);
+    int             val_log_highest_debug_level(void);
     const char     *val_get_ns_string(struct sockaddr *serv, char *dst,
                                       size_t size);
 
@@ -668,6 +703,15 @@ extern          "C" {
     void val_free_answer_chain(struct val_answer_chain *answers);
 
 
+    /*
+     * async functions from val_resquery.c
+     */
+    int val_async_select_info(val_context_t *context,
+                              fd_set *fds,
+                              int *num_fds,
+                              struct timeval *timeout);
+
+
     /*
      * for backwards compatibility
      */
