@@ -257,7 +257,7 @@ do_section(ns_msg * handle, ns_sect section, int pflag, FILE * file)
  * This is intended to be primarily a debugging routine.
  */
 void
-libsres_pquery(const u_char * msg, int len, FILE * file)
+libsres_pquery(const u_char * msg, size_t len, FILE * file)
 {
     ns_msg          handle;
     int             qdcount, ancount, nscount, arcount;
@@ -406,7 +406,7 @@ p_fqname(const u_char * cp, const u_char * msg, FILE * file)
  * And, for once, solaris has a better header than the rest, and has const
  * char ptrs in res_sym.
  */
-#if (defined(__p_class_syms) || defined(sun) || defined(__FreeBSD__) || defined(__NetBSD__) || defined(__OpenBSD__))
+#if (defined(__p_class_syms) || defined(sun) || defined(__FreeBSD__) || defined(__NetBSD__) || defined(__OpenBSD__) || defined(__CYGWIN__))
 #define RES_SYM_TYPE res_sym
 #else
 #define RES_SYM_TYPE res_sym_const
