@@ -58,27 +58,29 @@
 #define FALSE 0
 #endif
 
-static const int res_io_debug = FALSE;
+static int res_io_debug = FALSE;
+
+void
+res_io_set_debug(int val)
+{
+    res_io_debug = val;
+}
+
+int
+res_io_get_debug(void)
+{
+    return res_io_debug;
+}
 
 /*
  * Less than or equal comparison for timeval structures, ignoring
  * microseconds, just like res_send().
  */
 #define LTEQ(a,b)           (a.tv_sec<=b.tv_sec)
-#define MAX_TRANSACTIONS    128
-#define UPDATE(a,b) \
-        if (a->tv_sec==0 || !LTEQ((*a),b)) memcpy (a, &b, sizeof(struct timeval))
-
-#define MOVE_TO_NEXT_ADDRESS(t) \
-    if (t->ea_socket != -1) close (t->ea_socket); \
-    t->ea_socket = -1; \
-    t->ea_which_address++; \
-    t->ea_remaining_attempts = t->ea_ns->ns_retry+1; \
-    set_alarm(&(t->ea_next_try), 0); \
-    set_alarm(&(t->ea_cancel_time),res_timeout(t->ea_ns));
-
-#define MORE_ADDRESSES(ea) \
-    ea->ea_which_address < (ea->ea_ns->ns_number_of_addresses-1)
+#define UPDATE(a,b) do {                                                \
+    if (a->tv_sec==0 || !LTEQ((*a),b))                                  \
+        memcpy (a, &b, sizeof(struct timeval));                         \
+    } while(0)
 
 struct expected_arrival {
     int             ea_socket;
@@ -95,6 +97,7 @@ struct expected_arrival {
     struct expected_arrival *ea_next;
 };
 
+#define MAX_TRANSACTIONS    128
 static struct expected_arrival *transactions[MAX_TRANSACTIONS] = {
     NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
     NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
@@ -133,7 +136,7 @@ bind_to_random_source(int s)
     struct sockaddr_in *sa = (struct sockaddr_in *) &ea_source;
     u_int16_t next_port, start_port;
 
-    memset(sa, 0, sizeof(struct sockaddr_in));
+    memset(sa, 0, sizeof(ea_source));
     sa->sin_family = AF_INET;
     sa->sin_addr.s_addr = htonl(INADDR_ANY);
     start_port = (libsres_random() % 64512) + 1024;
@@ -159,7 +162,7 @@ bind_to_random_source(int s)
 }
 
 long
-res_timeout(struct name_server *ns)
+res_get_timeout(struct name_server *ns)
 {
     int             i;
     long            cancel_delay = 0;
@@ -193,6 +196,18 @@ res_sq_free_expected_arrival(struct expected_arrival **ea)
     *ea = NULL;
 }
 
+void
+res_free_ea_list(struct expected_arrival *head)
+{
+    struct expected_arrival *ea;
+
+    while (head) {
+        ea = head;
+        head = head->ea_next;
+        res_sq_free_expected_arrival(&ea);
+    }
+}
+
 void
 set_alarm(struct timeval *tv, long delay)
 {
@@ -214,6 +229,7 @@ res_ea_init(u_char * signed_query, size_t signed_length,
         /** We're out of memory */
         return NULL;
 
+    memset(temp, 0x0, sizeof(struct expected_arrival));
     temp->ea_socket = -1;
     temp->ea_ns = ns;
     temp->ea_which_address = 0;
@@ -224,12 +240,49 @@ res_ea_init(u_char * signed_query, size_t signed_length,
     temp->ea_response_length = 0;
     temp->ea_remaining_attempts = ns->ns_retry+1;
     set_alarm(&temp->ea_next_try, delay);
-    set_alarm(&temp->ea_cancel_time, delay + res_timeout(ns));
+    set_alarm(&temp->ea_cancel_time, delay + res_get_timeout(ns));
     temp->ea_next = NULL;
 
     return temp;
 }
 
+void
+res_io_cancel_remaining_attempts(struct expected_arrival *ea)
+{
+    if (ea->ea_socket != -1) {
+        close(ea->ea_socket);
+        ea->ea_socket = -1;
+    }
+    ea->ea_remaining_attempts = -1;
+}
+
+void
+res_io_cancel_all_remaining_attempts(struct expected_arrival *ea)
+{
+    for ( ; ea; ea = ea->ea_next) {
+        if (ea->ea_socket != -1) {
+            close(ea->ea_socket);
+            ea->ea_socket = -1;
+        }
+        ea->ea_remaining_attempts = -1;
+    }
+}
+
+int
+res_io_is_finished(struct expected_arrival *ea)
+{
+    return ea->ea_remaining_attempts == -1;
+}
+
+int
+res_io_are_all_finished(struct expected_arrival *ea)
+{
+    for ( ; ea; ea = ea->ea_next) 
+        if (ea->ea_remaining_attempts != -1)
+            return FALSE;
+    return TRUE;
+}
+
 int
 res_io_send(struct expected_arrival *shipit)
 {
@@ -240,10 +293,14 @@ res_io_send(struct expected_arrival *shipit)
     int             socket_type;
     size_t          socket_size;
     size_t          bytes_sent;
+    long            delay;
 
     if (shipit == NULL)
         return SR_IO_INTERNAL_ERROR;
 
+    if (res_io_debug)
+        printf("SENDING\n");
+
     socket_type = shipit->ea_using_stream ? SOCK_STREAM : SOCK_DGRAM;
 
     /*
@@ -331,6 +388,12 @@ res_io_send(struct expected_arrival *shipit)
         return SR_IO_SOCKET_ERROR;
     }
 
+    delay = shipit->ea_ns->ns_retrans
+        << (shipit->ea_ns->ns_retry + 1 - shipit->ea_remaining_attempts--);
+    set_alarm(&shipit->ea_next_try, delay);
+    if (res_io_debug)
+        res_print_ea(shipit);
+
     return SR_IO_UNSET;
 }
 
@@ -339,25 +402,41 @@ res_nsfallback(int transaction_id, struct timeval *closest_event,
                const char *name, const u_int16_t class_h, 
                const u_int16_t type_h, int *edns0)
 {
-    struct expected_arrival *temp, *t;
-    int i;
-    long delay = 0;
+    struct expected_arrival *temp;
+    int ret_val = -1;
 
-    const static int edns0_fallback[] = { 4096, 1492, 512, 0 };
-   
-    *edns0 = 0;
-
-    if (res_io_debug) {
-        printf("Aborting current attempt for transaction %d\n", transaction_id);
-    }
-    
-    pthread_mutex_lock(&mutex);
-    if (transaction_id == -1 || 
-        (temp = transactions[transaction_id]) == NULL) {
-        
-        pthread_mutex_unlock(&mutex);
+    if (transaction_id == -1 )
         return -1;  
+
+    pthread_mutex_lock(&mutex);
+    for (temp = transactions[transaction_id];
+         temp && temp->ea_remaining_attempts == -1;
+         temp = temp->ea_next)
+         ;
+    if (temp != NULL) {
+        if (res_io_debug)
+            printf("Aborting current attempt for transaction %d\n",
+                   transaction_id);
+        ret_val = res_nsfallback_ea(temp, closest_event, name, class_h, type_h,
+                                    edns0);
     }
+    else
+        *edns0 = 0;
+
+    pthread_mutex_unlock(&mutex);
+    return ret_val;
+}
+
+int 
+res_nsfallback_ea(struct expected_arrival *temp, struct timeval *closest_event, 
+                  const char *name, const u_int16_t class_h, 
+                  const u_int16_t type_h, int *edns0)
+{
+    const static int edns0_fallback[] = { 4096, 1492, 512, 0 };
+    long             delay = 0, i;
+
+    if (!temp && !name)
+        return -1;
 
     if ((temp->ea_ns->ns_options & RES_USE_DNSSEC) && 
         (temp->ea_ns->ns_edns0_size > 0)) {
@@ -382,7 +461,7 @@ res_nsfallback(int transaction_id, struct timeval *closest_event,
                         break;
                 }
                 temp->ea_remaining_attempts++;
-                if (temp->ea_socket)
+                if (temp->ea_socket != -1)
                     close(temp->ea_socket);
                 temp->ea_socket = -1;
 
@@ -390,11 +469,11 @@ res_nsfallback(int transaction_id, struct timeval *closest_event,
             }
         }
     }
+    else
+        *edns0 = 0;
 
-    if (temp->ea_remaining_attempts == 0) {
-        pthread_mutex_unlock(&mutex);
+    if (temp->ea_remaining_attempts == 0)
         return -1;
-    }
 
     gettimeofday(&temp->ea_next_try, NULL);
     for (i = 0; i < temp->ea_remaining_attempts; i++)
@@ -407,6 +486,7 @@ res_nsfallback(int transaction_id, struct timeval *closest_event,
      *  offset it to the current time 
      */
     if (temp->ea_next) {
+        struct expected_arrival *t;
         long offset = temp->ea_next->ea_next_try.tv_sec - 
                             temp->ea_next_try.tv_sec;
         if (offset > 0) {
@@ -416,19 +496,111 @@ res_nsfallback(int transaction_id, struct timeval *closest_event,
             } 
         }
     }
-    pthread_mutex_unlock(&mutex);
     return 0;
 }
 
+static void
+res_io_next_address(struct expected_arrival *ea,
+                    const char *more_prefix, const char *no_more_str)
+{
+    /*
+     * If there is another address, move to it else cancel it 
+     */
+    if (ea->ea_which_address < (ea->ea_ns->ns_number_of_addresses-1)) {
+        /*
+         * Start over with new address 
+         */
+        if (ea->ea_socket != -1) {
+            close (ea->ea_socket);
+            ea->ea_socket = -1;
+        }
+        ea->ea_which_address++;
+        ea->ea_remaining_attempts = ea->ea_ns->ns_retry+1;
+        set_alarm(&(ea->ea_next_try), 0);
+        set_alarm(&(ea->ea_cancel_time),res_get_timeout(ea->ea_ns));
+        if (res_io_debug) {
+            printf("%s - SWITCHING TO NEW ADDRESS\n", more_prefix);
+            res_print_ea(ea);
+        }
+    } else {
+        /*
+         * cancel this source 
+         */
+        res_io_cancel_remaining_attempts(ea);
+        if (res_io_debug) {
+            printf("%s\n", no_more_str);
+            res_print_ea(ea);
+        }
+    }
+}
+
+int
+res_io_check_one(struct expected_arrival *ea, struct timeval *next_evt,
+                 struct timeval *now)
+{
+    int             total = 0;
+    struct timeval  local_now;
+    
+    /*
+     * if caller didn't pass us current time, get it
+     */
+    if (NULL == now) {
+        now = &local_now;
+        gettimeofday(&local_now, NULL);
+        local_now.tv_usec = 0;
+    }
+
+    for ( ; ea; ea = ea->ea_next ) {
+        if (ea->ea_remaining_attempts == -1)
+            continue;
+
+        if (res_io_debug)
+            printf("res_io_check_one socket=%d\n", ea->ea_socket);
+
+        /*
+         * check for timeouts. If there is another address, move to it
+         */
+        if ( LTEQ(ea->ea_cancel_time, (*now))) {
+            res_io_next_address(ea, "TIMEOUTS", "TIMEOUT - CANCELING");
+            --total;
+        }
+
+        /*
+         * send next try. on error, if there is another address, move to it
+         */
+        else if (LTEQ(ea->ea_next_try, (*now))) {
+            int needed_new_socket = (ea->ea_socket == -1);
+            while (ea->ea_remaining_attempts != -1) {
+                if (res_io_send(ea) == SR_IO_SOCKET_ERROR) {
+                    res_io_next_address(ea, "ERROR",
+                                        "CANCELING DUE TO SENDING ERROR");
+                }
+                else {
+                    if (needed_new_socket)
+                        ++total;
+                    break; /* from while remaining attempts */
+                }
+            } /* while */
+        }
+
+        /*
+         * update next event
+         */
+        if (next_evt && ea->ea_remaining_attempts != -1) {
+            UPDATE(next_evt, ea->ea_cancel_time);
+            UPDATE(next_evt, ea->ea_next_try);
+        }
+    }
+
+    return total;
+}
+
 static int
 res_io_check(int transaction_id, struct timeval *next_evt)
 {
     int             i;
-    int             total = 0;
     struct timeval  tv;
-    struct expected_arrival *temp1;
-    struct expected_arrival *temp2;
-    long            delay;
+    struct expected_arrival *ea;
 
     gettimeofday(&tv, NULL);
     tv.tv_usec = 0;
@@ -443,127 +615,24 @@ res_io_check(int transaction_id, struct timeval *next_evt)
 
     pthread_mutex_lock(&mutex);
 
-    for (i = 0; i < MAX_TRANSACTIONS; i++) {
-        while (transactions[i]
-               && LTEQ(transactions[i]->ea_cancel_time, tv)) {
-            if (res_io_debug) {
-                printf("res_io_check trans_id=%d\n", i);
-            }
-            /*
-             * If there is another address, move to it else cancel it 
-             */
-            if (MORE_ADDRESSES(transactions[i])) {
-                /*
-                 * Start over with new address 
-                 */
-                MOVE_TO_NEXT_ADDRESS(transactions[i]);
-                if (res_io_debug) {
-                    printf("TIMEOUTS - SWITCHING TO NEW ADDRESS (*)\n");
-                    res_print_ea(transactions[i]);
-                }
-            } else {
-                /*
-                 * Retire this source 
-                 */
-                temp1 = transactions[i];
-                transactions[i] = transactions[i]->ea_next;
-                if (res_io_debug) {
-                    printf("TIMEOUT - CANCELING (*)\n");
-                    res_print_ea(temp1);
-                }
-                res_sq_free_expected_arrival(&temp1);
-            }
-        }
-
-        if (transactions[i]) {
-            temp1 = transactions[i];
-
-            do {
-                if (LTEQ(temp1->ea_next_try, tv)
-                    && temp1->ea_remaining_attempts) {
-                    if (res_io_debug)
-                        printf("SENDING\n");
-                    if (res_io_send(temp1) == SR_IO_SOCKET_ERROR) {
-                        /*
-                         * If there is another address, move to it
-                         * else cancel it 
-                         */
-                        if (MORE_ADDRESSES(temp1)) {
-                            /*
-                             * Start over with new address 
-                             */
-                            MOVE_TO_NEXT_ADDRESS(temp1);
-                            if (res_io_debug) {
-                                printf
-                                    ("ERROR - SWITCHING TO NEW ADDRESS\n");
-                                res_print_ea(temp1);
-                            }
-                        } else {
-                            if (res_io_debug) {
-                                res_print_ea(temp1);
-                                printf("CANCELING DUE TO SENDING ERROR\n");
-                            }
-                            /*
-                             * Cancel this particular source 
-                             */
-                            temp1->ea_remaining_attempts = 0;
-                            gettimeofday(&temp1->ea_cancel_time, NULL);
-                            if (res_io_debug)
-                                res_print_ea(temp1);
-                        }
-                    } else {
-                        if (i == transaction_id)
-                            total++;
-                        delay = temp1->ea_ns->ns_retrans
-                            << (temp1->ea_ns->ns_retry + 1 -
-                                temp1->ea_remaining_attempts--);
-                        set_alarm(&temp1->ea_next_try, delay);
-                        if (res_io_debug)
-                            res_print_ea(temp1);
-                    }
-                } else if (i == transaction_id)
-                    total++;
-
-                UPDATE(next_evt, temp1->ea_cancel_time);
-                UPDATE(next_evt, temp1->ea_next_try);
-
-                while (temp1->ea_next &&
-                       LTEQ(temp1->ea_next->ea_cancel_time, tv)) {
-                    /*
-                     * Same logic as before, if there are more
-                     * addresses for this source try them, else
-                     * cancel the source.
-                     */
-                    if (MORE_ADDRESSES(temp1->ea_next)) {
-                        /*
-                         * Start over with new address 
-                         */
-                        MOVE_TO_NEXT_ADDRESS(temp1->ea_next);
-                        if (res_io_debug) {
-                            printf("TIMEOUT - SWITCHING TO NEW ADDRESS\n");
-                            res_print_ea(temp1->ea_next);
-                        }
-                    } else {
-                        if (res_io_debug) {
-                            printf("TIMEOUT - CANCELING\n");
-                            res_print_ea(temp1->ea_next);
-                        }
-                        temp2 = temp1->ea_next;
-                        temp1->ea_next = temp1->ea_next->ea_next;
-                        res_sq_free_expected_arrival(&temp2);
-                    }
-                }
-
-                temp1 = temp1->ea_next;
-            } while (temp1);
-        }
-    }
+    for (i = 0; i < MAX_TRANSACTIONS; i++)
+        if (transactions[i])
+            res_io_check_one(transactions[i], next_evt, &tv);
 
+    ea = transactions[transaction_id];
     pthread_mutex_unlock(&mutex);
 
     if (res_io_debug)
         printf("Next event is at %ld\n", next_evt->tv_sec);
-    return total;
+
+    /*
+     * check for remaining attempts for this transaction
+     */
+    for (; ea; ea = ea->ea_next)
+        if (ea->ea_remaining_attempts != -1)
+            return 1;
+
+    return 0; /* no events for this transaction */
 }
 
 int
@@ -571,7 +640,7 @@ res_io_deliver(int *transaction_id, u_char * signed_query,
                size_t signed_length, struct name_server *ns, long delay)
 {
     int             try_index;
-    struct expected_arrival *temp;
+    struct expected_arrival *temp, *new_ea;
     struct timeval  next_event;
 
     /*
@@ -605,17 +674,17 @@ res_io_deliver(int *transaction_id, u_char * signed_query,
     /*
      * Register this request 
      */
+    new_ea = res_ea_init(signed_query, signed_length, ns, delay);
+    if (new_ea == NULL) {
+        /** We can't add this */
+        pthread_mutex_unlock(&mutex);
+        return SR_IO_MEMORY_ERROR;
+    }
     if (transactions[*transaction_id] == NULL) {
         /*
          * Add this as the first request 
          */
-        if ((transactions[*transaction_id] =
-             res_ea_init(signed_query, signed_length, ns,
-                         delay)) == NULL) {
-            /** We can't add this */
-            pthread_mutex_unlock(&mutex);
-            return SR_IO_MEMORY_ERROR;
-        }
+        transactions[*transaction_id] = new_ea;
     } else {
         /*
          * Retaining order is important 
@@ -623,12 +692,7 @@ res_io_deliver(int *transaction_id, u_char * signed_query,
         temp = transactions[*transaction_id];
         while (temp->ea_next)
             temp = temp->ea_next;
-        if ((temp->ea_next =
-             res_ea_init(signed_query, signed_length, ns,
-                         delay)) == NULL) {
-            pthread_mutex_unlock(&mutex);
-            return SR_IO_MEMORY_ERROR;
-        }
+        temp->ea_next = new_ea;
     }
 
     pthread_mutex_unlock(&mutex);
@@ -658,21 +722,41 @@ void
 res_io_collect_sockets(fd_set * read_descriptors,
                        struct expected_arrival *ea_list)
 {
+    res_io_select_info(ea_list, NULL, read_descriptors, NULL);
+}
+
+void
+res_io_select_info(struct expected_arrival *ea_list, int *nfds,
+                   fd_set * read_descriptors, struct timeval *timeout)
+{
+    struct timeval now;
+
+    if (timeout) {
+        gettimeofday(&now, NULL);
+        now.tv_usec = 0;
+    }
     /*
      * Find all sockets in use for a particular transaction chain of
      * expected arrivals
      */
-    //FD_ZERO(read_descriptors);
+    for ( ; ea_list; ea_list = ea_list->ea_next) {
+        if ((ea_list->ea_remaining_attempts == -1) ||
+            (ea_list->ea_socket == -1))
+            continue;
 
-    while (ea_list) {
-        if (ea_list->ea_socket != -1)
+        if (read_descriptors)
             FD_SET(ea_list->ea_socket, read_descriptors);
+        if (nfds && (ea_list->ea_socket >= *nfds))
+            *nfds = ea_list->ea_socket + 1;
 
-        ea_list = ea_list->ea_next;
+        if (timeout) {
+            UPDATE(timeout, ea_list->ea_cancel_time);
+            UPDATE(timeout, ea_list->ea_next_try);
+        }
     }
 }
 
-int
+static int
 res_io_select_sockets(fd_set * read_descriptors, struct timeval *timeout)
 {
     /*
@@ -695,14 +779,22 @@ res_io_select_sockets(fd_set * read_descriptors, struct timeval *timeout)
     if (max_sock == -1)
         return 0; /* nothing to read */
 
+    if (res_io_debug) {
+        printf("select on %d fds, timeout %ld sec\n", max_sock + 1,
+               timeout->tv_sec);
+        fflush(stdout);
+    }
+#ifdef HAVE_PSELECT
     timeout_ts.tv_sec = timeout->tv_sec;
     timeout_ts.tv_nsec = 0;
-
-#ifdef HAVE_PSELECT
-    return pselect(max_sock + 1, read_descriptors, NULL, NULL, &timeout_ts, NULL);
+    i = pselect(max_sock + 1, read_descriptors, NULL, NULL, &timeout_ts, NULL);
 #else
-    return select(max_sock + 1, read_descriptors, NULL, NULL, timeout);
+    i = select(max_sock + 1, read_descriptors, NULL, NULL, timeout);
 #endif
+    if (res_io_debug)
+        printf("got %d fds\n", i);
+
+    return i;
 }
 
 void
@@ -732,19 +824,38 @@ res_io_get_a_response(struct expected_arrival *ea_list, u_char ** answer,
                       struct name_server **respondent)
 {
     int             retval;
+    int             save_count = -1;
 
-    while (ea_list) {
-        if (ea_list->ea_response) {
+    for( ; ea_list; ea_list = ea_list->ea_next) {
+        if (ea_list->ea_remaining_attempts == -1)
+            continue;
+
+        if (!ea_list->ea_response)
+            continue;
+
+        { /** dummy block to preserve indentation; reformat later */
             *answer = ea_list->ea_response;
             *answer_length = ea_list->ea_response_length;
+            if (res_io_debug)
+                printf("get_response got %d bytes on socket %d\n",
+                       *answer_length, ea_list->ea_socket);
+
+            /*
+             * don't clone all when we just need one. temporarily set
+             * number of nameservers to 1 before cloning.
+             */
+            if (ea_list->ea_ns->ns_number_of_addresses > 1) {
+                save_count = ea_list->ea_ns->ns_number_of_addresses;
+                ea_list->ea_ns->ns_number_of_addresses = 1;
+            }
             if (SR_UNSET !=
                 (retval = clone_ns(respondent, ea_list->ea_ns)))
                 return retval;
-            if ((*respondent)->ns_number_of_addresses > 0) {
-                /*
-                 * fix the actual server 
-                 */
-                (*respondent)->ns_number_of_addresses = 1;
+            if (save_count > 0) /* restore original count */
+                ea_list->ea_ns->ns_number_of_addresses = save_count;
+
+            /** if response wasn't from first address, fixup respondent */
+            if (ea_list->ea_which_address != 0) {
                 memcpy(((*respondent)->ns_address[0]),
                        ea_list->ea_ns->ns_address[ea_list->
                                                   ea_which_address],
@@ -754,7 +865,6 @@ res_io_get_a_response(struct expected_arrival *ea_list, u_char ** answer,
             ea_list->ea_response_length = 0;
             return SR_IO_GOT_ANSWER;
         }
-        ea_list = ea_list->ea_next;
     }
     return SR_IO_UNSET;
 }
@@ -791,16 +901,13 @@ res_io_read_tcp(struct expected_arrival *arrival)
 
     if (complete_read(arrival->ea_socket, arrival->ea_response, len_h) !=
         len_h) {
-        close(arrival->ea_socket);
-        arrival->ea_socket = -1;
         FREE(arrival->ea_response);
         arrival->ea_response = NULL;
         arrival->ea_response_length = 0;
         /*
          * Cancel this source 
          */
-        arrival->ea_remaining_attempts = 0;
-        gettimeofday(&arrival->ea_cancel_time, NULL);
+        res_io_cancel_remaining_attempts(arrival);
         return SR_IO_SOCKET_ERROR;
     }
     return SR_IO_UNSET;
@@ -809,7 +916,7 @@ res_io_read_tcp(struct expected_arrival *arrival)
 int
 res_io_read_udp(struct expected_arrival *arrival)
 {
-    size_t bytes_waiting = 0;
+    size_t bytes_waiting = 8192;
     struct sockaddr_storage from;
     socklen_t       from_length = sizeof(from);
     int             ret_val, arr_family;
@@ -817,14 +924,6 @@ res_io_read_udp(struct expected_arrival *arrival)
     if (NULL == arrival)
         return SR_IO_INTERNAL_ERROR;
 
-    if (ioctl(arrival->ea_socket, FIONREAD, &bytes_waiting) == -1) {
-        close(arrival->ea_socket);
-        if (res_io_debug)
-            printf("Closing socket %d, FIONREAD failed\n", arrival->ea_socket);
-        arrival->ea_socket = -1;
-        return SR_IO_SOCKET_ERROR;
-    }
-
     arrival->ea_response = (u_char *) MALLOC(bytes_waiting * sizeof(u_char));
     if (NULL == arrival->ea_response)
         return SR_IO_MEMORY_ERROR;
@@ -833,6 +932,17 @@ res_io_read_udp(struct expected_arrival *arrival)
         recvfrom(arrival->ea_socket, arrival->ea_response, bytes_waiting,
                  0, (struct sockaddr*)&from, &from_length);
 
+    if (0 == ret_val) { /* what does 0 bytes mean from udp socket? */
+        // xxx-rks: allow other attempt, or goto error?
+        if (res_io_debug)
+            printf("0 bytes on socket %d, read_udp failed\n",
+                   arrival->ea_socket);
+        FREE(arrival->ea_response);
+        arrival->ea_response = NULL;
+        arrival->ea_response_length = 0;
+        return SR_IO_SOCKET_ERROR;
+    }
+
     arr_family = arrival->ea_ns->ns_address[arrival->ea_which_address]->ss_family;
     if ((ret_val < 0) || (from.ss_family != arr_family))
         goto error;
@@ -867,18 +977,15 @@ res_io_read_udp(struct expected_arrival *arrival)
     return SR_IO_UNSET;
 
   error:
-    close(arrival->ea_socket);
     if (res_io_debug)
         printf("Closing socket %d, read_udp failed\n", arrival->ea_socket);
-    arrival->ea_socket = -1;
     FREE(arrival->ea_response);
     arrival->ea_response = NULL;
     arrival->ea_response_length = 0;
     /*
      * Cancel this source 
      */
-    arrival->ea_remaining_attempts = 0;
-    gettimeofday(&arrival->ea_cancel_time, NULL);
+    res_io_cancel_remaining_attempts(arrival);
     return SR_IO_SOCKET_ERROR;
 }
 
@@ -900,40 +1007,37 @@ res_switch_to_tcp(struct expected_arrival *ea)
      * Use the same "ea_which_address," since it already got a rise. 
      */
     ea->ea_using_stream = TRUE;
-    ea->ea_socket = -1;
+    if (ea->ea_socket != -1) {
+        close(ea->ea_socket);
+        ea->ea_socket = -1;
+    }
     ea->ea_remaining_attempts = ea->ea_ns->ns_retry+1;
     set_alarm(&ea->ea_next_try, 0);
 
-    set_alarm(&ea->ea_cancel_time, res_timeout(ea->ea_ns));
+    set_alarm(&ea->ea_cancel_time, res_get_timeout(ea->ea_ns));
 }
 
-void
+int
 res_io_read(fd_set * read_descriptors, struct expected_arrival *ea_list)
 {
-    int             sock, i;
+    int             handled = 0;
     struct expected_arrival *arrival;
 
-    i = getdtablesize(); 
-    if (i > FD_SETSIZE)
-        i = FD_SETSIZE;
-    for (sock = 0; sock < i; ++sock)
-        if (FD_ISSET(sock, read_descriptors)) {
-            if (res_io_debug)
-                printf("ACTIVITY on %d\n", sock);
-            /*
-             * This socket is ready for reading 
-             */
-            arrival = ea_list;
-            while (arrival && arrival->ea_socket != sock)
-                arrival = arrival->ea_next;
+    for (; ea_list; ea_list = ea_list->ea_next) {
+        /*
+         * skip canceled/expired attempts, or sockets without data
+         */
+        if ((ea_list->ea_remaining_attempts == -1) ||
+            (ea_list->ea_socket == -1) ||
+            ! FD_ISSET(ea_list->ea_socket, read_descriptors))
+            continue;
 
-            if (arrival == NULL) {
-                if (res_io_debug)
-                    printf
-                        ("Ummm, we lost the record that this socket belongs to.\nSorry.\n");
-                close(sock);
-                continue;
-            }
+        { /* dummy block to preserve indentation; remove later */
+
+            if (res_io_debug)
+                printf("ACTIVITY on %d\n", ea_list->ea_socket);
+            ++handled;
+            arrival = ea_list;
             if (res_io_debug)
                 res_print_ea(arrival);
 
@@ -941,17 +1045,14 @@ res_io_read(fd_set * read_descriptors, struct expected_arrival *ea_list)
                 /** Use TCP */
                 if (res_io_read_tcp(arrival) == SR_IO_SOCKET_ERROR)
                     continue;
-
-                if (res_io_debug)
-                    printf("Read via TCP\n");
             } else {
                 /** Use UDP */
                 if (res_io_read_udp(arrival) == SR_IO_SOCKET_ERROR)
                     continue;
-
-                if (res_io_debug)
-                    printf("Read via UDP\n");
             }
+            if (res_io_debug)
+                printf("Read %d byptes via %s\n", arrival->ea_response_length,
+                       arrival->ea_using_stream ? "TCP" : "UDP");
 
             /*
              * Make sure this is the query we want (buffer id's match).
@@ -987,7 +1088,8 @@ res_io_read(fd_set * read_descriptors, struct expected_arrival *ea_list)
                 && ((HEADER *) arrival->ea_response)->tc)
                 res_switch_to_tcp(arrival);
         }
-    return;
+    }
+    return handled;
 }
 
 int
@@ -1030,7 +1132,6 @@ res_io_accept(int transaction_id, fd_set *pending_desc,
         pthread_mutex_unlock(&mutex);
         return SR_IO_GOT_ANSWER;
     }
-    pthread_mutex_unlock(&mutex);
 
     /*
      * Decision time: does this call only look at the sockets used by
@@ -1038,7 +1139,6 @@ res_io_accept(int transaction_id, fd_set *pending_desc,
      * 
      * Answer for now -> just the sockets we are interested in.
      */
-    pthread_mutex_lock(&mutex);
     res_io_collect_sockets(&read_descriptors, 
                            transactions[transaction_id]);
     pthread_mutex_unlock(&mutex);
@@ -1049,34 +1149,37 @@ res_io_accept(int transaction_id, fd_set *pending_desc,
         /** select call failed */
         return SR_IO_SOCKET_ERROR;
 
+    pthread_mutex_lock(&mutex);
+
+    if (transactions[transaction_id] == NULL) {
+        pthread_mutex_unlock(&mutex);
+        return SR_IO_NO_ANSWER;
+    }
+
     if (ret_val == 0) { 
         /** There are sources, but none are talking (yet) */
 
         /* save descriptors that we are waiting on */
-        pthread_mutex_lock(&mutex);
         res_io_collect_sockets(pending_desc, 
                                transactions[transaction_id]);
-        pthread_mutex_unlock(&mutex);
 
         /* check if next_event is closer than closest_event */
         if (closest_event->tv_sec == 0 ||
             LTEQ(next_event, (*closest_event))) {
             closest_event->tv_sec = next_event.tv_sec;
         }
+        pthread_mutex_unlock(&mutex);
         return SR_IO_NO_ANSWER_YET;
     }
 
     /*
      * React to the active desciptors.
      */
-    pthread_mutex_lock(&mutex);
     res_io_read(&read_descriptors, transactions[transaction_id]);
-    pthread_mutex_unlock(&mutex);
 
     /*
      * Pluck the answer and return it to the caller.
      */
-    pthread_mutex_lock(&mutex);
     ret_val = res_io_get_a_response(transactions[transaction_id],
                                     answer, answer_length, respondent);
     pthread_mutex_unlock(&mutex);
@@ -1096,14 +1199,12 @@ res_cancel(int *transaction_id)
         return;
 
     pthread_mutex_lock(&mutex);
-    while (transactions[*transaction_id]) {
-        ea = transactions[*transaction_id];
-        transactions[*transaction_id] =
-            transactions[*transaction_id]->ea_next;
-        res_sq_free_expected_arrival(&ea);
-    }
+    ea = transactions[*transaction_id];
+    transactions[*transaction_id] = NULL;
     pthread_mutex_unlock(&mutex);
 
+    res_free_ea_list(ea);
+
     *transaction_id = -1;
 }
 
@@ -1190,3 +1291,133 @@ res_io_stall(void)
 
     sleep(100 - (tv.tv_sec % 100));
 }
+
+struct expected_arrival *
+res_async_query_send(const char *name, const u_int16_t type_h,
+                     const u_int16_t class_h, struct name_server *pref_ns)
+{
+    int                 ret_val = SR_UNSET;
+    u_char             *signed_query = NULL;
+    size_t              signed_length = 0;
+    struct name_server *ns_list = NULL;
+    struct name_server *ns;
+    struct expected_arrival *head = NULL, *new_ea, *temp_ea;
+    long                delay = 0;
+
+    if ((name == NULL) || (pref_ns == NULL))
+        return NULL;
+
+    /*
+     * clone nameservers and store to ns_list
+     */
+    if ((ret_val = clone_ns_list(&ns_list, pref_ns)) != SR_UNSET)
+        return NULL;
+
+    /*
+     * Loop through the list of destinations, form the query and send it
+     */
+    for (ns = ns_list; ns; ns = ns->ns_next) {
+
+        /** create payload */
+        ret_val = res_create_query_payload(ns, name, class_h, type_h,
+                                           &signed_query, &signed_length);
+        if (ret_val < 0)
+            break; /* fatal, bail */
+
+        /** create expected arrival struct */
+        new_ea = res_ea_init(signed_query, signed_length, ns, delay);
+        if (NULL == new_ea) {
+            FREE(signed_query);
+            ret_val = SR_IO_MEMORY_ERROR;
+            break; /* fatal, bail */
+        }
+
+        /** add to list */
+        if (NULL != head) {
+            temp_ea = head;
+            while(temp_ea->ea_next)
+                temp_ea = temp_ea->ea_next;
+            temp_ea->ea_next = new_ea;
+        } else
+            head = new_ea;
+
+        delay += LIBSRES_NS_STAGGER;
+    }
+
+    /** if bad ret_val, clear list, else send query */
+    if (ret_val != SR_UNSET) {
+        res_free_ea_list(head);
+        head = NULL;
+    }
+    else
+        ret_val = res_io_check_one(head,NULL,NULL);
+
+    return head;
+}
+
+void
+res_async_query_select_info(struct expected_arrival *ea, int *nfds,
+                            fd_set *fds, struct timeval *timeout)
+{
+    if (!ea || (!nfds && !fds && !timeout))
+        return;
+
+    res_io_select_info(ea, nfds, fds, timeout);
+}
+
+int
+res_async_query_handle(struct expected_arrival *ea, int *handled, fd_set *fds)
+{
+    int ret_val = SR_NO_ANSWER;
+
+    if (!ea || !handled || !fds)
+        return SR_INTERNAL_ERROR;
+
+    /*
+     * React to any active desciptors and see if we got a response, or
+     * if we at least still have an open socket (i.e. potential response).
+     */
+    *handled = res_io_read(fds, ea);
+    for( ; ea; ea = ea->ea_next) {
+        if (ea->ea_remaining_attempts == -1)
+            continue;
+        else if (ea->ea_response) {
+            ret_val = SR_UNSET;
+            break;
+        }
+        else if (ea->ea_socket != -1)
+            ret_val = SR_NO_ANSWER_YET;
+    }
+
+    return ret_val;
+}
+
+void
+res_async_query_free(struct expected_arrival *ea)
+{
+    res_free_ea_list(ea);
+}
+
+void
+res_async_query_cancel(struct expected_arrival *ea)
+{
+
+}
+
+int
+res_async_ea_is_using_stream(struct expected_arrival *ea)
+{
+    if (NULL == ea)
+        return 0;
+
+    return ea->ea_using_stream;
+}
+
+int
+res_async_ea_isset(struct expected_arrival *ea, fd_set *fds)
+{
+    if (NULL == ea || NULL == fds)
+        return 0;
+
+    return FD_ISSET(ea->ea_socket, fds);
+}
