@@ -16,7 +16,7 @@
  * WITH THE USE OR PERFORMANCE OF THE SOFTWARE.
  */
 /*
- * Copyright 2005-2009 SPARTA, Inc.  All rights reserved.
+ * Copyright 2005-2011 SPARTA, Inc.  All rights reserved.
  * See the COPYING file distributed with this software for details.
  */
 #include "validator-config.h"
@@ -371,8 +371,8 @@ query_send(const char *name,
 {
     int             ret_val;
 
-    u_char         *signed_query;
-    size_t          signed_length;
+    u_char         *signed_query = NULL;
+    size_t          signed_length = 0;
 
     struct name_server *ns_list = NULL;
     struct name_server *ns;
@@ -496,6 +496,7 @@ get(const char *name,
                 wait_for_res_data(&pending_desc, &closest_event);
             }
         } while (ret_val == SR_NO_ANSWER_YET);
+        res_cancel(&trans_id); /* cleanup transaction */
     }
 
     return ret_val;
