@@ -25,13 +25,13 @@
 #define SR_TS_FAIL              -2
 #define SR_TS_CALL_ERROR        -3
 
-int             res_tsig_sign(u_int8_t * query,
-                              int query_length,
+int             res_tsig_sign(u_char * query,
+                              size_t query_length,
                               struct name_server *ns,
-                              u_int8_t ** signed_query,
-                              int *signed_length);
+                              u_char ** signed_query,
+                              size_t *signed_length);
 
 int             res_tsig_verifies(struct name_server *respondent,
-                                  u_int8_t * answer, int answer_length);
+                                  u_char * answer, size_t answer_length);
 
 #endif
