@@ -16,7 +16,7 @@
  * WITH THE USE OR PERFORMANCE OF THE SOFTWARE.
  */
 /*
- * Copyright 2005-2009 SPARTA, Inc.  All rights reserved.
+ * Copyright 2005-2011 SPARTA, Inc.  All rights reserved.
  * See the COPYING file distributed with this software for details.
  */
 /*
@@ -35,6 +35,7 @@
 #include <pthread.h>
 #endif
 
+#include <arpa/nameser.h>
 #include <validator/resolver.h>
 #include <validator/validator.h>
 #include <validator/validator-internal.h>
@@ -125,6 +126,10 @@ stow_info(struct rrset_rec **unchecked_info, struct rrset_rec **new_info, struct
     if (new_info == NULL || unchecked_info == NULL)
         return VAL_NO_ERROR;
 
+    if (matched_q->qc_flags & VAL_QUERY_REFRESH_QCACHE) {
+        matched_q->qc_flags ^= VAL_QUERY_REFRESH_QCACHE;
+    }
+
     trail_new = NULL;
     prev = NULL;
     while (*new_info) {
@@ -209,6 +214,9 @@ get_cached_rrset(struct val_query_chain *matched_q,
     u_int16_t class_h;
     u_char *name_n;
 
+    int look_for_negative;
+    int done;
+
     if (!matched_q || !response)
         return VAL_BAD_ARGUMENT;
 
@@ -218,79 +226,81 @@ get_cached_rrset(struct val_query_chain *matched_q,
     type_h = matched_q->qc_type_h;
     class_h = matched_q->qc_class_h;
 
-
     gettimeofday(&tv, NULL);
 
-    switch (type_h) {
+    look_for_negative = 0;
+    done = 0;
 
-    case ns_t_ns:
-        lk = &ns_rwlock;
-        LOCK_INIT(lk, ns_rwlock_init);
-        LOCK_SH(lk);
-        answer_head = &unchecked_ns_info;
-        break;
+    while (!done) {
 
-    default:
-        lk = &ans_rwlock;
-        LOCK_INIT(lk, ans_rwlock_init);
-        LOCK_SH(lk);
-        answer_head = &unchecked_answers;
-        break;
-    }
-
-    prev = NULL;
-    new_answer = NULL;
-    if (answer_head) 
-        next_answer = *answer_head;
-    else
-        next_answer = NULL;
-    
-    while (next_answer) {
-
-        if (tv.tv_sec >= next_answer->rrs_ttl_x) {
-            // TTL expiry reached
-            struct rrset_rec *temp;
-            temp = next_answer;
-            next_answer = temp->rrs_next;
-            temp->rrs_next = NULL;
-            if (prev) {
-                prev->rrs_next = next_answer;
-            } else { 
-                *answer_head = next_answer;
-            }
-            res_sq_free_rrset_recs(&temp);
-            continue;
+        if (look_for_negative) {
+            lk = &proof_rwlock;
+            LOCK_INIT(lk, proof_rwlock_init);
+            LOCK_SH(lk);
+            answer_head = &unchecked_proofs;
+        } else if (type_h == ns_t_ns) {
+            lk = &ns_rwlock;
+            LOCK_INIT(lk, ns_rwlock_init);
+            LOCK_SH(lk);
+            answer_head = &unchecked_ns_info;
+        } else {
+            lk = &ans_rwlock;
+            LOCK_INIT(lk, ans_rwlock_init);
+            LOCK_SH(lk);
+            answer_head = &unchecked_answers;
         }
 
-        if (
-            next_answer->rrs_class_h == class_h) {
+        prev = NULL;
+        new_answer = NULL;
+        if (answer_head) 
+            next_answer = *answer_head;
+        else
+            next_answer = NULL;
+    
+        while (next_answer) {
+
+            if (tv.tv_sec < next_answer->rrs_ttl_x &&
+                next_answer->rrs_class_h == class_h) {
 
                 /* if matching type or cname indirection */
-            if (((next_answer->rrs_type_h == type_h ||
-                 (next_answer->rrs_type_h == ns_t_cname &&
-                  type_h != ns_t_any && type_h != ns_t_rrsig)) &&
-                /* and name is an exact match */
-                (namecmp(next_answer->rrs_name_n, name_n) == 0)) ||
-                /* OR */
-                /* DNAME indirection */
-                ((next_answer->rrs_type_h == ns_t_dname &&
-                  type_h != ns_t_any && type_h != ns_t_rrsig) &&
-                /* and name applies */
-                 (NULL != (u_char *) namename(name_n, 
+                if (((next_answer->rrs_type_h == type_h ||
+                    (next_answer->rrs_type_h == ns_t_cname &&
+                    ALIAS_MATCH_TYPE(type_h))) &&
+                    /* and name is an exact match */
+                    (namecmp(next_answer->rrs_name_n, name_n) == 0)) ||
+                    /* OR */
+                    /* DNAME indirection */
+                    ((next_answer->rrs_type_h == ns_t_dname &&
+                    ALIAS_MATCH_TYPE(type_h)) &&
+                    /* and name applies */
+                    (NULL != (u_char *) namename(name_n, 
                                     next_answer->rrs_name_n)))) {
 
-                if (next_answer->rrs_data != NULL) {
-                    new_answer = copy_rrset_rec(next_answer);
-                    break;
-                }
-            } 
+                    if (next_answer->rrs_data != NULL) {
+                        new_answer = copy_rrset_rec(next_answer);
+                        if (new_answer) {
+                            /* Adjust the TTL */
+                            new_answer->rrs_ttl_h = next_answer->rrs_ttl_x - tv.tv_sec; 
+                        }
+                        break;
+                    }
+                } 
+            }
+
+            prev = next_answer;
+            next_answer = next_answer->rrs_next;
         }
 
-        prev = next_answer;
-        next_answer = next_answer->rrs_next;
-    }
+        UNLOCK(lk);
 
-    UNLOCK(lk);
+        if (!new_answer && type_h == ns_t_soa && !look_for_negative) {
+            /* look for soa in negative cache */
+            done = 0;
+            look_for_negative = 1;
+        } else {
+            done = 1;
+        }
+    }
 
     /* Construct the response */
     if (new_answer) {
@@ -556,22 +566,7 @@ get_nslist_from_cache(val_context_t *ctx,
     nsrrset = unchecked_ns_info;
     while (nsrrset) {
 
-        if (tv.tv_sec >= nsrrset->rrs_ttl_x) {
-            /* TTL expiry reached */
-            struct rrset_rec *temp;
-            temp = nsrrset;
-            nsrrset = temp->rrs_next;
-            temp->rrs_next = NULL;
-            if (prev) {
-                prev->rrs_next = nsrrset;
-            } else { 
-                unchecked_ns_info = nsrrset;
-            }
-            res_sq_free_rrset_recs(&temp);
-            continue;
-        }
-
-        if (
+        if (tv.tv_sec < nsrrset->rrs_ttl_x &&
             nsrrset->rrs_type_h == ns_t_ns) {
 
             tname_n = nsrrset->rrs_name_n;
