@@ -1,5 +1,5 @@
 /*
- * Copyright 2005-2009 SPARTA, Inc.  All rights reserved.
+ * Copyright 2005-2011 SPARTA, Inc.  All rights reserved.
  * See the COPYING file distributed with this software for details.
  */
 /*
@@ -18,6 +18,7 @@
 #include <pthread.h>
 #endif
 
+#include <arpa/nameser.h>
 #include <validator/resolver.h>
 #include <validator/validator.h>
 #include <validator/validator-internal.h>
@@ -27,6 +28,20 @@
 #include "val_assertion.h"
 #include "val_context.h"
 
+#define GET_LATEST_TIMESTAMP(ctx, file, cur_ts, new_ts) do { \
+    memset(&new_ts, 0, sizeof(struct stat));\
+    if (!file) {\
+        if (cur_ts != 0) {\
+            val_log(ctx, LOG_WARNING, "val_resolve_and_check(): %s missing; trying to operate without it.", file);\
+        }\
+    } else {\
+        if(0 != stat(file, &new_ts)) {\
+            val_log(ctx, LOG_WARNING, "val_resolve_and_check(): %s missing; trying to operate without it.", file);\
+        }\
+    }\
+}while (0)
+
+
 static val_context_t *the_default_context = NULL;
 
 #ifndef VAL_NO_THREADS
@@ -232,6 +247,8 @@ unlink_if_default_context(val_context_t *context)
 void
 val_free_context(val_context_t * context)
 {
+    struct val_query_chain *q;
+
     if (context == NULL)
         return;
 
@@ -266,11 +283,63 @@ val_free_context(val_context_t * context)
     destroy_valpol(context);
     FREE(context->e_pol);
 
-    free_query_chain(context->q_list);
+    while (NULL != (q = context->q_list)) {
+        context->q_list = q->qc_next;
+        free_query_chain_structure(q);
+    }
+
+#ifndef VAL_NO_ASYNC
+    {
+        val_async_status  *as;
+        while (NULL != context->as_list) {
+            as = context->as_list;
+            context->as_list = as->val_as_next;
+            as->val_as_next = NULL;
+            as->val_as_ctx = NULL;
+            val_async_status_free(as);
+        }
+    }
+#endif
 
     FREE(context);
 }
 
+#ifndef VAL_NO_ASYNC
+/* remove asynchronous status from context async queries list */
+int
+val_context_as_remove(val_context_t *context, val_async_status *as)
+{
+    val_async_status *prev, *curr = NULL;
+
+    if ((NULL == context) || (NULL == as) ||
+        (as->val_as_ctx && (as->val_as_ctx != context)))
+        return VAL_BAD_ARGUMENT;
+
+    if (NULL == context->as_list)
+        return VAL_NO_ERROR;
+
+    if (as == context->as_list) {
+        curr = context->as_list;
+        context->as_list = as->val_as_next;
+    } else {
+        prev = context->as_list;
+        curr = prev->val_as_next;
+        for (; curr; prev = curr, curr = curr->val_as_next) {
+            if (curr != as)
+                continue;
+            prev->val_as_next = curr->val_as_next;
+            break;
+        } /* for */
+    } /* as->val_as_ctx */
+
+    if (curr) {
+        curr->val_as_next = NULL;
+        curr->val_as_ctx = NULL;
+    }
+    return VAL_NO_ERROR;
+}
+#endif
+
 /*
  * Free all internal state associated with the validator
  * Only used when testing if we have memory leaks
@@ -368,3 +437,97 @@ val_refresh_root_hints(val_context_t * context)
     return VAL_NO_ERROR;
 }
 
+/*
+ * Function: val_refresh_context
+ *
+ * Purpose:   set up context for a query
+ *
+ * Parameter: results -- results for query
+ *            ctx -- user supplied context, if any
+ *
+ * Returns:   VAL_NO_ERROR or error code to return to user
+ *
+ */
+int
+val_refresh_context(val_context_t *context)
+{
+    struct stat rsb, vsb, hsb;
+    struct dnsval_list *dnsval_l;
+    int retval;
+
+    if (NULL == context)
+        return VAL_BAD_ARGUMENT;
+
+    CTX_LOCK_RESPOL_SH(context);
+
+    GET_LATEST_TIMESTAMP(context, context->resolv_conf, context->r_timestamp,
+                         rsb);
+    if (rsb.st_mtime != 0 &&  rsb.st_mtime != context->r_timestamp) {
+        CTX_UNLOCK_RESPOL(context);
+        if (VAL_NO_ERROR != (retval = val_refresh_resolver_policy(context)))
+            return retval;
+        CTX_LOCK_RESPOL_SH(context);
+    }
+
+    GET_LATEST_TIMESTAMP(context, context->root_conf, context->h_timestamp, hsb);
+    if (hsb.st_mtime != 0 &&  hsb.st_mtime != context->h_timestamp){
+        CTX_UNLOCK_RESPOL(context);
+        if (VAL_NO_ERROR != (retval = val_refresh_root_hints(context)))
+            return retval;
+        CTX_LOCK_RESPOL_SH(context);
+    }
+
+    CTX_LOCK_VALPOL_SH(context);
+    /* dnsval.conf can point to a list of files */
+    for (dnsval_l = context->dnsval_l; dnsval_l; dnsval_l=dnsval_l->next) {
+        GET_LATEST_TIMESTAMP(context,  dnsval_l->dnsval_conf, 
+                             dnsval_l->v_timestamp, vsb);
+        if (vsb.st_mtime != 0 &&  vsb.st_mtime != dnsval_l->v_timestamp) {
+            CTX_UNLOCK_VALPOL(context);
+            retval = val_refresh_validator_policy(context);
+            if (VAL_NO_ERROR != retval) {
+                CTX_UNLOCK_RESPOL(context);
+                return retval;
+            }
+            CTX_LOCK_VALPOL_SH(context);
+            break;
+        }
+    }
+    CTX_UNLOCK_VALPOL(context);
+    CTX_UNLOCK_RESPOL(context);
+
+    return VAL_NO_ERROR;
+}
+
+/*
+ * Function: val_async_setup_context
+ *
+ * Purpose:   set up context for a query
+ *
+ * Parameter: results -- results for query
+ *            ctx -- user supplied context, if any
+ *
+ * Returns:   VAL_NO_ERROR or error code to return to user
+ *
+ */
+val_context_t *
+val_create_or_refresh_context(val_context_t *ctx)
+{
+    val_context_t *context;
+    int retval;
+
+    /*
+     * Create a default context if one does not exist
+     */
+    if (ctx == NULL) {
+        if (VAL_NO_ERROR != (retval = val_create_context(NULL, &context)))
+            return NULL;
+    } else {
+        /* have configuration files changed? */
+        context = ctx;
+        if (VAL_NO_ERROR != (retval = val_refresh_context(context)))
+            return NULL;
+    }
+
+    return context;
+}
