@@ -1,5 +1,5 @@
 /*
- * Copyright 2005-2009 SPARTA, Inc.  All rights reserved.
+ * Copyright 2005-2011 SPARTA, Inc.  All rights reserved.
  * See the COPYING file distributed with this software for details.
  *
  * Author: Abhijit Hayatnagarkar
@@ -33,6 +33,7 @@
 #include <ctype.h>
 #include <stdlib.h>
 
+#include <arpa/nameser.h>
 #include <validator/resolver.h>
 #include <validator/validator.h>
 #include <validator/validator-internal.h>
@@ -477,6 +478,7 @@ ds_sha_hash_is_equal(u_char * name_n,
     SHA1_Update(&c, qc_name_n, namelen);
     SHA1_Update(&c, rrdata, rrdatalen);
     SHA1_Final(ds_digest, &c);
+    FREE(qc_name_n);
 
     if (!memcmp(ds_digest, ds_hash, SHA_DIGEST_LENGTH))
         return 1;
@@ -516,6 +518,7 @@ ds_sha256_hash_is_equal(u_char * name_n,
     SHA256_Update(&c, qc_name_n, namelen);
     SHA256_Update(&c, rrdata, rrdatalen);
     SHA256_Final(ds_digest, &c);
+    FREE(qc_name_n);
 
     if (!memcmp(ds_digest, ds_hash, SHA256_DIGEST_LENGTH))
         return 1;
