@@ -1,5 +1,5 @@
 /*
- * Copyright 2005-2009 SPARTA, Inc.  All rights reserved.
+ * Copyright 2005-2011 SPARTA, Inc.  All rights reserved.
  * See the COPYING file distributed with this software for details.
  *
  * Author: Abhijit Hayatnagarkar
@@ -1166,8 +1166,7 @@ val_getnameinfo(val_context_t * context,
                 size_t servlen, int flags, val_status_t * val_status)
 {
 
-    const int       addr_size = 100;
-    char            domain_string[addr_size], number_string[addr_size];
+    char            domain_string[NS_MAXDNAME], number_string[NS_MAXDNAME];
     const u_char   *theAddress = NULL;
     int             theAddressFamily;
     int             val_rnc_status = 0, ret_status = 0;
@@ -1190,6 +1189,8 @@ val_getnameinfo(val_context_t * context,
             serv == NULL ? "(null)" : serv);
 
 
+    *val_status = VAL_UNTRUSTED_ANSWER;
+
     /*
      * check misc parameters, there should be at least one of host or
      * server, check if flags indicate host is required 
@@ -1197,8 +1198,6 @@ val_getnameinfo(val_context_t * context,
     if (!val_status || !sa)
       return EAI_FAIL;
 
-    *val_status = VAL_UNTRUSTED_ANSWER;
-
     if (!host && !serv) 
       return EAI_NONAME;
     
@@ -1207,26 +1206,43 @@ val_getnameinfo(val_context_t * context,
      */
     if (serv && servlen > 0) {
         struct servent *sent;
-        int port = ((const struct sockaddr_in*)sa)->sin_port;
+        u_int16_t port;
+        if (sa->sa_family == AF_INET) {
+            port = ((const struct sockaddr_in*)sa)->sin_port;
+        } else if (sa->sa_family == AF_INET6) {
+            port = ((const struct sockaddr_in6*)sa)->sin6_port;
+        } else {
+            val_log(ctx, LOG_DEBUG, "val_getnameinfo(): Address family %d not known.", sa->sa_family);
+            return EAI_FAMILY;
+        }
 
         val_log(ctx, LOG_DEBUG, 
-            "val_getnameinfo(): get service for port(%d)\n",ntohs(port));
+            "val_getnameinfo(): get service for port(%d)",ntohs(port));
         if (flags & NI_DGRAM)
             sent = getservbyport(port, "udp");
         else
             sent = getservbyport(port, NULL);
 
-        if (!sent)
-            return EAI_FAIL;
-
-        if (flags & NI_NUMERICSERV) {
-            val_log(ctx, LOG_DEBUG, "val_getnameinfo(): NI_NUMERICSERV\n");
-            snprintf(serv, servlen, "%d", ntohs(sent->s_port));
-        } else {
-            strncpy(serv, sent->s_name, servlen);
-        }
-        val_log(ctx, LOG_DEBUG, "val_getnameinfo(): service is %s : %s ",
+        if (sent) {
+            if (flags & NI_NUMERICSERV) {
+                val_log(ctx, LOG_DEBUG, "val_getnameinfo(): NI_NUMERICSERV");
+                snprintf(serv, servlen, "%d", ntohs(sent->s_port));
+            } else {
+                strncpy(serv, sent->s_name, servlen);
+            }
+            val_log(ctx, LOG_DEBUG, "val_getnameinfo(): service is %s : %s ",
                 serv, sent->s_proto);
+        } else {
+            strncpy(serv, "", servlen);
+        }
+    }
+
+    /*
+     * should the host be looked up 
+     */
+    if (!host || hostlen == 0) {
+        *val_status = VAL_TRUSTED_ANSWER;
+        return 0;
     }
 
     /*
@@ -1246,7 +1262,7 @@ val_getnameinfo(val_context_t * context,
         if (!(flags & NI_NUMERICHOST) &&
             (0 == memcmp(&((const struct sockaddr_in6 *) sa)->sin6_addr,
                          _ipv6_wrapped_ipv4, sizeof(_ipv6_wrapped_ipv4)))) {
-            val_log(ctx, LOG_DEBUG, "val_getnameinfo(): ipv4 wrapped addr\n");
+            val_log(ctx, LOG_DEBUG, "val_getnameinfo(): ipv4 wrapped addr");
             theAddress += sizeof(_ipv6_wrapped_ipv4);
             theAddressFamily = AF_INET;
         }
@@ -1259,40 +1275,36 @@ val_getnameinfo(val_context_t * context,
     }
 
     /*
-     * should the host be looked up 
+     * get string values: address string, reverse domain string, on
+     * the wire reverse domain string 
      */
-    if (host && hostlen > 0) {
-        /*
-         * get string values: address string, reverse domain string, on
-         * the wire reverse domain string 
-         */
-        memset(number_string, 0, sizeof(number_string));
-        memset(domain_string, 0, sizeof(domain_string));
+    memset(number_string, 0, sizeof(number_string));
+    memset(domain_string, 0, sizeof(domain_string));
 
-        if ((0 != (ret_status =
+    if ((0 != (ret_status =
              address_to_string(theAddress, theAddressFamily,
                      number_string, sizeof(number_string))))
-              ||
-            (0 != (ret_status =
-              address_to_reverse_domain(theAddress, theAddressFamily,
+         ||
+        (0 != (ret_status =
+             address_to_reverse_domain(theAddress, theAddressFamily,
                       domain_string, sizeof(domain_string))))) {
             return ret_status;
-        }
-
-        /*
-         * set numeric value initially for either NI_NUMERICHOST or failed lookup
-         */
-        strncpy(host, number_string, hostlen);
-        *val_status = VAL_TRUSTED_ANSWER;
     }
 
-    val_log(ctx, LOG_DEBUG, "val_getnameinfo(): pre-val flags(%d)\n", flags);
+    /*
+     * set numeric value initially for either NI_NUMERICHOST or failed lookup
+     */
+    strncpy(host, number_string, hostlen);
+
+    val_log(ctx, LOG_DEBUG, "val_getnameinfo(): pre-val flags(%d)", flags);
 
     if ((flags & NI_NUMERICHOST) && !(flags & NI_NAMEREQD)) {
+        *val_status = VAL_TRUSTED_ANSWER;
+        val_log(ctx, LOG_DEBUG, "val_getnameinfo(): returning host (%s)", host);
         return 0;
     }
 
-    val_log(ctx, LOG_DEBUG, "val_getnameinfo(): val_get_rrset host flags(%d)\n", flags);
+    val_log(ctx, LOG_DEBUG, "val_getnameinfo(): val_get_rrset host flags(%d)", flags);
     if (VAL_NO_ERROR != 
               (retval = val_get_rrset(ctx,       /*val_context_t*  */
                                       domain_string, /*u_char *wire_domain_name */
@@ -1308,7 +1320,7 @@ val_getnameinfo(val_context_t * context,
     }
 
     if (!val_res) {
-        val_log(ctx, LOG_ERR, "val_getnameinfo(): EAI_MEMORY\n");
+        val_log(ctx, LOG_ERR, "val_getnameinfo(): EAI_MEMORY");
         *val_status = VAL_UNTRUSTED_ANSWER;
         return EAI_MEMORY;
     }
@@ -1341,18 +1353,14 @@ val_getnameinfo(val_context_t * context,
             break;
         } 
     }
+    val_free_answer_chain(val_res);
 
-    if (res == NULL) {
-        *val_status = VAL_UNTRUSTED_ANSWER;
-    }
-                    
     val_log(ctx, LOG_DEBUG,
           "val_getnameinfo(): val_get_rrset for host %s, returned %s with lookup status %d and validator status %d : %s",
           domain_string, host,
           val_rnc_status, 
           *val_status, p_val_status(*val_status));
 
-    val_free_answer_chain(val_res);
 
     return val_rnc_status;
 
