@@ -1,5 +1,5 @@
 /*
- * Copyright 2005-2009 SPARTA, Inc.  All rights reserved.
+ * Copyright 2005-2011 SPARTA, Inc.  All rights reserved.
  * See the COPYING file distributed with this software for details.
  *
  * Author: Abhijit Hayatnagarkar
@@ -23,6 +23,7 @@
 #include <strings.h>
 #include <errno.h>
 
+#include <arpa/nameser.h>
 #include <validator/validator.h>
 #include <validator/resolver.h>
 #include "val_policy.h"
@@ -308,6 +309,7 @@ get_hostent_from_response(val_context_t * ctx, int af, struct hostent *ret,
     int validated = 1;
     int trusted = 1;
     struct val_rrset_rec *rrset;
+    char *alias_target = NULL;
 
     /*
      * Check parameter sanity 
@@ -387,7 +389,6 @@ get_hostent_from_response(val_context_t * ctx, int af, struct hostent *ret,
      */
     for (res = results; res != NULL; res = res->val_rc_next) {
 
-        char *alias_target = NULL;
         rrset = res->val_rc_rrset;
 
         if (!(validated && val_isvalidated(res->val_rc_status))) 
@@ -463,20 +464,17 @@ get_hostent_from_response(val_context_t * ctx, int af, struct hostent *ret,
                     rr = rr->rr_next;
                 }
             }
-
-        } else {
-            /* pick up official name from the alias target */
-            if (!ret->h_name) {
-                ret->h_name = (char *) bufalloc(buf, buflen, offset,
-                                              (strlen(rrset->val_rrset_name) +
-                                               1) * sizeof(char));
-                if (ret->h_name == NULL) {
-                    goto err;
-                }
-                memcpy(ret->h_name, rrset->val_rrset_name, strlen(rrset->val_rrset_name) + 1);
-            }
         }
     }
+    /* pick up official name from the alias target */
+    if (!ret->h_name && alias_target) {
+        ret->h_name = (char *) bufalloc(buf, buflen, offset,
+                          (strlen(alias_target) + 1) * sizeof(char));
+        if (ret->h_name == NULL) {
+            goto err;
+        }
+        memcpy(ret->h_name, alias_target, strlen(alias_target) + 1);
+    }
 
     if (addr_count > 0) {
         *h_errnop = NETDB_SUCCESS;
@@ -689,7 +687,7 @@ val_gethostbyname2_r(val_context_t * context,
         if (VAL_NO_ERROR ==
                 (retval =
                  val_resolve_and_check(ctx, name, ns_c_in, type,
-                                       VAL_QUERY_NO_AC_DETAIL,
+                                       0,
                                        &results))) {
 
             /*
@@ -854,8 +852,7 @@ val_gethostbyaddr_r(val_context_t * context,
                     int *h_errnop, val_status_t * val_status)
 {
 
-    const int       addr_size = 100;
-    char            domain_string[addr_size];
+    char            domain_string[NS_MAXDNAME];
     int             ret_status = 0, bufused = 0;
     struct val_answer_chain *val_res = NULL;
     struct val_answer_chain *res;
@@ -905,11 +902,11 @@ val_gethostbyaddr_r(val_context_t * context,
         return (NO_RECOVERY);
     }
 
-    memset(domain_string, 0, sizeof(char) * addr_size);
+    memset(domain_string, 0, sizeof(domain_string));
 
     if (0 !=
         (ret_status = address_to_reverse_domain(addr, type,
-                                                domain_string, addr_size))
+                                                domain_string, sizeof(domain_string)))
         ) {
         *h_errnop = ret_status;
         return ret_status;
