@@ -1,6 +1,6 @@
 
 /*
- * Copyright 2005-2009 SPARTA, Inc.  All rights reserved.
+ * Copyright 2005-2011 SPARTA, Inc.  All rights reserved.
  * See the COPYING file distributed with this software for details.
  */
 /*
@@ -44,6 +44,18 @@ val_log_set_debug_level(int level)
     debug_level = level;
 }
 
+int
+val_log_highest_debug_level(void)
+{
+    val_log_t      *tmp_log;
+    int             level = 0;
+
+    for (tmp_log = default_log_head; tmp_log; tmp_log = tmp_log->next)
+        if (tmp_log->level > level)
+            level = tmp_log->level;
+
+    return level;
+}
 
 char           *
 get_hex_string(const u_char *data, size_t datalen, char *buf,
@@ -657,9 +669,12 @@ val_log_insert(val_log_t **log_head, val_log_t * logp)
 {
     val_log_t      *tmp_log;
 
-    if (NULL == logp || NULL == log_head)
+    if (NULL == logp)
         return;
 
+    if (log_head == NULL)
+        log_head = &default_log_head;
+
     for (tmp_log = *log_head; tmp_log && tmp_log->next;
          tmp_log = tmp_log->next);
 
