@@ -1,5 +1,5 @@
 /*
- * Copyright 2005-2009 SPARTA, Inc.  All rights reserved.
+ * Copyright 2005-2011 SPARTA, Inc.  All rights reserved.
  * See the COPYING file distributed with this software for details.
  */
 /*
@@ -356,6 +356,8 @@ free_trust_anchor(policy_entry_t * pol_entry)
                 FREE(ta_pol->publickey->public_key);
             FREE(ta_pol->publickey);
         } else if (ta_pol->ds) {
+            if (ta_pol->ds->d_hash)
+                FREE(ta_pol->ds->d_hash);
             FREE(ta_pol->ds);
         }
         FREE(ta_pol);
@@ -820,9 +822,9 @@ parse_local_answer_gopt(char **buf_ptr, char *end_ptr, int *line_number,
         return VAL_CONF_PARSE_ERROR;
     }
 
-    if (!strcmp(token, TRUST_OOB_GOPT_YES_STR))
+    if (!strcmp(token, GOPT_YES_STR))
         g_opt->local_is_trusted = 1;
-    else if (!strcmp(token, TRUST_OOB_GOPT_NO_STR))
+    else if (!strcmp(token, GOPT_NO_STR))
         g_opt->local_is_trusted = 0;
     else
         return VAL_CONF_PARSE_ERROR;
@@ -922,6 +924,39 @@ parse_log_target_gopt(char **buf_ptr, char *end_ptr, int *line_number,
     return VAL_NO_ERROR;
 }
 
+static int
+parse_closest_ta_target_gopt(char **buf_ptr, char *end_ptr, int *line_number,
+                      int *endst, global_opt_t *g_opt)
+{
+    char            token[TOKEN_MAX];
+    int retval;
+
+    if ((buf_ptr == NULL) || (*buf_ptr == NULL) || (end_ptr == NULL) || 
+        (g_opt == NULL) || (endst == NULL) || (line_number == NULL))
+        return VAL_BAD_ARGUMENT;
+
+    /* read the next token */
+    if (VAL_NO_ERROR != (retval = 
+        val_get_token(buf_ptr, end_ptr, line_number, 
+                      token, sizeof(token), endst,
+                      CONF_COMMENT, CONF_END_STMT, 0))) {
+        return retval;
+    }
+    if ((endst && (strlen(token) == 0)) ||
+        (*buf_ptr >= end_ptr)) { 
+        return VAL_CONF_PARSE_ERROR;
+    }
+
+    if (!strncmp(token, GOPT_YES_STR, strlen(GOPT_YES_STR))) {
+        g_opt->closest_ta_only = 1;
+    } else if (!strncmp(token, GOPT_NO_STR, strlen(GOPT_NO_STR))) {
+        g_opt->closest_ta_only = 0;
+    } else {
+        return VAL_CONF_PARSE_ERROR;
+    }
+    return VAL_NO_ERROR;
+}
+
 static int
 get_global_options(char **buf_ptr, char *end_ptr, 
                    int *line_number, global_opt_t **g_opt) 
@@ -942,6 +977,7 @@ get_global_options(char **buf_ptr, char *end_ptr,
     (*g_opt)->env_policy = VAL_POL_GOPT_DISABLE;
     (*g_opt)->app_policy = VAL_POL_GOPT_DISABLE;
     (*g_opt)->log_target = NULL;
+    (*g_opt)->closest_ta_only = 0;
     
     while (!endst) {
         /*
@@ -993,6 +1029,13 @@ get_global_options(char **buf_ptr, char *end_ptr,
                                                     line_number, &endst, *g_opt))) {
                 goto err;
             }
+        } else if (!strcmp(token, GOPT_CLOSEST_TA_ONLY_STR)) {
+            if (VAL_NO_ERROR != 
+                    (retval = parse_closest_ta_target_gopt(buf_ptr, end_ptr, 
+                                                    line_number, &endst, *g_opt))) {
+                goto err;
+            }
+
         } else {
             retval = VAL_CONF_PARSE_ERROR;
             goto err;
@@ -1040,6 +1083,10 @@ get_next_policy_fragment(char **buf_ptr, char *end_ptr, char *scope,
             free_policy_entry(pol, index);
             pol = NULL;
         }
+        if (label != NULL) {
+            FREE(label);
+            label = NULL;
+        }
 
         if (VAL_NO_ERROR !=
             (retval =
@@ -1050,8 +1097,6 @@ get_next_policy_fragment(char **buf_ptr, char *end_ptr, char *scope,
             return VAL_NO_ERROR;
         if (endst)
             return VAL_CONF_PARSE_ERROR;
-        if (label != NULL)
-            FREE(label);
         label = (char *) MALLOC(strlen(token) + 1);
         if (label == NULL)
             return VAL_OUT_OF_MEMORY;
@@ -1333,7 +1378,7 @@ destroy_valpol(val_context_t * ctx)
     }
 
     if (ctx->g_opt) {
-        /*XX should stop logging to current channel */
+        /*XXX should stop logging to current channel */
         free_global_options(ctx->g_opt);
         FREE(ctx->g_opt);
         ctx->g_opt = NULL;
@@ -1351,7 +1396,7 @@ static int
 read_next_val_config_file(val_context_t *ctx, 
                           char **label, 
                           struct dnsval_list *dnsval_c, 
-                          struct dnsval_list *dnsval_list, 
+                          struct dnsval_list *dlist, 
                           struct dnsval_list **added_files,
                           struct policy_overrides **overrides,
                           global_opt_t **g_opt)
@@ -1375,7 +1420,7 @@ read_next_val_config_file(val_context_t *ctx,
     char *env = NULL;
 
     if (ctx == NULL || label == NULL || dnsval_c == NULL || 
-        dnsval_list == NULL || added_files == NULL || 
+        dlist == NULL || added_files == NULL || 
         overrides == NULL || g_opt == NULL)
         return VAL_BAD_ARGUMENT;
 
@@ -1391,7 +1436,7 @@ read_next_val_config_file(val_context_t *ctx,
                 "read_next_val_config_file(): Could not open validator conf file for reading: %s",
                 dnsval_c->dnsval_conf);
         /* check if we have at least one file */
-        if (dnsval_c == dnsval_list) {
+        if (dnsval_c == dlist) {
             retval = VAL_CONF_NOT_FOUND;
             goto err;
         } else {
@@ -1430,25 +1475,19 @@ read_next_val_config_file(val_context_t *ctx,
 
     while (!done) {
 
-        if (pol_frag) {
-            FREE(pol_frag->label);
-            free_policy_entry(pol_frag->pol, pol_frag->index);
-            FREE(pol_frag);
-            pol_frag = NULL;
-        }
-
         if (*added_files) {
             FREE_DNSVAL_FILE_LIST(*added_files);
             *added_files = NULL;
         }
 
-        /* if we're looping again */
-        if (*overrides && dnsval_c == dnsval_list) {
-            /* trash our temporary overrides */
+        /* if we're looping again for the first file, trash our temporary overrides */
+        if (*overrides && dnsval_c == dlist) {
             destroy_valpolovr(overrides);
             *overrides = NULL;
         }
 
+        /* don't free global options g_opt since we're going to reuse this */
+
         buf_ptr = buf;
         end_ptr = buf+sb.st_size;
         dnsval_l = NULL;
@@ -1461,12 +1500,6 @@ read_next_val_config_file(val_context_t *ctx,
                                              &line_number, 
                                              &g_opt_seen,
                                              &include_seen))) {
-            if (buf_ptr >= end_ptr) {
-                /* done reading file */
-                retval = VAL_NO_ERROR;
-                break;
-            }
-            
             if (g_opt_seen) {
                 /* next policy fragment contains global options */ 
                 global_opt_t *gt_opt = NULL;
@@ -1480,7 +1513,7 @@ read_next_val_config_file(val_context_t *ctx,
                     goto err;
                 }
     
-                if (*g_opt || (dnsval_c != dnsval_list)) {
+                if (*g_opt || (dnsval_c != dlist)) {
                     /* 
                      * re-definition of global options 
                      * or global options was not in the first file
@@ -1511,11 +1544,16 @@ read_next_val_config_file(val_context_t *ctx,
                     if (gt_opt->app_policy == VAL_POL_GOPT_OVERRIDE ||
                             (*label == NULL && gt_opt->app_policy == VAL_POL_GOPT_ENABLE)) {
                         next_label = getprogname(); 
-                        val_log(ctx, LOG_NOTICE, 
-                                "read_next_val_config_file(): Using policy label from app name: %s",
-                                next_label);
-                        done = 0;
-                        break;
+                        if (next_label != NULL) {
+                            val_log(ctx, LOG_NOTICE, 
+                                    "read_next_val_config_file(): Using policy label from app name: %s",
+                                    next_label);
+                            done = 0;
+                            break;
+                        }
+                        /* policy does not exist, dont create the impression that we have one */
+                        gt_opt->app_policy = VAL_POL_GOPT_DISABLE;
+                        next_label = *label;
                     }
                 } 
             } else if (include_seen) { 
@@ -1606,7 +1644,7 @@ read_next_val_config_file(val_context_t *ctx,
                 } 
     
                 /* check if filename already exists in the list */
-                for (dnsval_temp=dnsval_list; dnsval_temp; dnsval_temp=dnsval_temp->next) {
+                for (dnsval_temp=dlist; dnsval_temp; dnsval_temp=dnsval_temp->next) {
                     if (!strcmp(dnsval_temp->dnsval_conf, token)) {
                         val_log(ctx, LOG_ERR, 
                                 "read_next_val_config_file(): File already included, possible loop in line %d of %s ",
@@ -1643,6 +1681,11 @@ read_next_val_config_file(val_context_t *ctx,
                 store_policy_overrides(ctx, overrides, &pol_frag);
                 pol_frag = NULL;
             }
+            if (buf_ptr >= end_ptr) {
+                /* done reading file */
+                retval = VAL_NO_ERROR;
+                break;
+            }
         }
     }
 
@@ -1690,11 +1733,12 @@ read_val_config_file(val_context_t * ctx, char *scope, int *is_override)
     struct policy_overrides *t;
     char *base_dnsval_conf = NULL;
     global_opt_t *g_opt = NULL;
-    struct dnsval_list *dnsval_list = NULL;
+    struct dnsval_list *dlist = NULL;
     struct dnsval_list *dnsval_c;
     int             retval;
     char *label;
     char *newctxlab;
+    struct val_query_chain *q;
    
     if (ctx == NULL || is_override == NULL)
         return VAL_BAD_ARGUMENT;
@@ -1708,14 +1752,14 @@ read_val_config_file(val_context_t * ctx, char *scope, int *is_override)
         return VAL_OUT_OF_MEMORY;
    
     /* create a new head element for the dnsval.conf file list */
-    dnsval_list = (struct dnsval_list *) MALLOC (sizeof(struct dnsval_list));
-    if (dnsval_list == NULL) {
+    dlist = (struct dnsval_list *) MALLOC (sizeof(struct dnsval_list));
+    if (dlist == NULL) {
         FREE(base_dnsval_conf);
         return VAL_OUT_OF_MEMORY;
     }
-    dnsval_list->dnsval_conf = base_dnsval_conf;
-    dnsval_list->next = NULL; 
-    dnsval_c = dnsval_list;
+    dlist->dnsval_conf = base_dnsval_conf;
+    dlist->next = NULL; 
+    dnsval_c = dlist;
     label = scope;
 
     overrides = NULL;
@@ -1729,7 +1773,7 @@ read_val_config_file(val_context_t * ctx, char *scope, int *is_override)
                 read_next_val_config_file(ctx,
                                           &label,
                                           dnsval_c,
-                                          dnsval_list,
+                                          dlist,
                                           &added_list,
                                           &overrides, 
                                           &g_opt))) {
@@ -1823,10 +1867,11 @@ read_val_config_file(val_context_t * ctx, char *scope, int *is_override)
     /* 
      * Re-initialize caches 
      */
-    free_query_chain(ctx->q_list);
+    for(q = ctx->q_list; q; q=q->qc_next) {
+        clear_query_chain_structure(q);
+    }
 
-    ctx->q_list = NULL;
-    ctx->dnsval_l = dnsval_list;
+    ctx->dnsval_l = dlist;
 
     CTX_UNLOCK_VALPOL(ctx);
 
@@ -1844,7 +1889,7 @@ err:
         FREE(g_opt);
         g_opt = NULL;
     }
-    FREE_DNSVAL_FILE_LIST(dnsval_list);
+    FREE_DNSVAL_FILE_LIST(dlist);
     return retval;
 }
 
@@ -1857,112 +1902,6 @@ destroy_respol(val_context_t * ctx)
     }
 }
 
-static int
-parse_name_server(char *cp, struct name_server **ns)
-{ 
-    short port_num = DNS_PORT;
-    char *cpt, addr[IPADDR_STRING_MAX];
-
-    struct sockaddr_storage serv_addr;
-    struct sockaddr_in *sin = (struct sockaddr_in *)&serv_addr;
-#ifdef VAL_IPV6
-    struct sockaddr_in6 *sin6 = (struct sockaddr_in6 *)&serv_addr;
-#endif
-    union {
-        struct in_addr   v4;
-#ifdef VAL_IPV6
-        struct in6_addr  v6;
-#endif
-    } address;
-
-    if (cp ==  NULL || ns == NULL)
-        return VAL_BAD_ARGUMENT;
-
-    *ns = (struct name_server *) MALLOC(sizeof(struct name_server));
-    if (*ns == NULL)
-        return VAL_OUT_OF_MEMORY; 
-
-    if (ns_name_pton(DEFAULT_ZONE, (*ns)->ns_name_n, 
-                sizeof((*ns)->ns_name_n)) == -1) {
-        FREE(*ns);
-        *ns = NULL;
-        return VAL_CONF_PARSE_ERROR; 
-    }
-
-    /*
-     * Initialize the rest of the fields 
-     */
-    (*ns)->ns_tsig = NULL;
-    (*ns)->ns_security_options = ZONE_USE_NOTHING;
-    (*ns)->ns_status = 0;
-
-    (*ns)->ns_retrans = RES_TIMEOUT;
-    (*ns)->ns_retry = RES_RETRY;
-    (*ns)->ns_options = RES_DEFAULT | RES_RECURSE | RES_DEBUG;
-    (*ns)->ns_edns0_size = RES_EDNS0_DEFAULT;
-
-    (*ns)->ns_next = NULL;
-    (*ns)->ns_number_of_addresses = 0;
-
-    /*
-     * Look for port number in address string
-     * syntax of '[address]:port'
-     */
-    cpt = cp;
-    if ( (*cpt == '[') && (cpt = strchr(cpt,']')) ) {
-      if ( IPADDR_STRING_MAX < (cpt - cp) ) {
-	goto parse_err;
-      }
-      bzero(addr, IPADDR_STRING_MAX);
-      strncpy(addr, (cp + 1), (cpt - cp - 1));
-      cp = addr;
-      if ( (*(++cpt) = ':') && (0 == (port_num = atoi(++cpt))) ) {
-	  goto parse_err;
-      }
-    }
-
-    /*
-     * convert address string
-     */
-    bzero(&serv_addr, sizeof(serv_addr));
-    if (inet_pton(AF_INET, cp, &address.v4) > 0) {
-        sin->sin_family = AF_INET;     // host byte order
-        sin->sin_addr = address.v4;
-        sin->sin_port = htons(port_num);       // short, network byte order
-    }
-    else {
-#ifdef VAL_IPV6
-        if (inet_pton(AF_INET6, cp, &address.v6) != 1)
-            goto parse_err;
-
-        sin6->sin6_family = AF_INET6;     // host byte order
-        memcpy(&sin6->sin6_addr, &address.v6, sizeof(address.v6));
-        sin6->sin6_port = htons(port_num);       // short, network byte order
-#else
-        goto parse_err;
-#endif
-    }
-
-    (*ns)->ns_address = NULL;
-    CREATE_NSADDR_ARRAY((*ns)->ns_address, 1);
-    if ((*ns)->ns_address == NULL) {
-        FREE(*ns);
-        *ns = NULL;
-        return VAL_OUT_OF_MEMORY;
-    }
-    (*ns)->ns_number_of_addresses = 1;
-
-    memcpy((*ns)->ns_address[0], &serv_addr,
-           sizeof(serv_addr));
-    (*ns)->ns_number_of_addresses = 1;
-    return VAL_NO_ERROR;
-
-  parse_err:
-    FREE(*ns);
-    *ns = NULL;
-    return VAL_CONF_PARSE_ERROR;
-}
-
 int
 read_res_config_file(val_context_t * ctx)
 {
@@ -2021,6 +1960,8 @@ read_res_config_file(val_context_t * ctx)
         retval = VAL_CONF_NOT_FOUND;
         goto err;
     } 
+    if (0 == sb.st_size)
+        goto empty;
 
     buf = (char *) MALLOC (sb.st_size * sizeof(char));
     if (buf == NULL) {
@@ -2067,8 +2008,7 @@ read_res_config_file(val_context_t * ctx)
 			"read_res_config_file(): error getting nameserver token!");
                 goto err;
             }
-            ns = NULL;
-            if (VAL_NO_ERROR != parse_name_server(token, &ns)) {
+            if ((ns = parse_name_server(token, DEFAULT_ZONE)) == NULL) {
                 val_log(ctx, LOG_WARNING,
 			"read_res_config_file(): error parsing nameserver token!");
                 goto err;
@@ -2096,8 +2036,7 @@ read_res_config_file(val_context_t * ctx)
                            ALL_COMMENTS, ZONE_END_STMT, 0))) {
                 goto err;
             }
-            ns = NULL;
-            if (VAL_NO_ERROR != parse_name_server(token, &ns))
+            if ((ns = parse_name_server(token, DEFAULT_ZONE)) == NULL)
                 goto err;
             /* zone next */
             if (VAL_NO_ERROR !=
@@ -2131,6 +2070,7 @@ read_res_config_file(val_context_t * ctx)
     }
 
     FREE(buf);
+  empty:
     fl.l_type = F_UNLCK;
     fcntl(fd, F_SETLKW, &fl);
     close(fd);
@@ -2662,14 +2602,12 @@ val_add_valpolicy(val_context_t *context,
 
     /* Flush queries that match this name */
     for(q=ctx->q_list; q; q=q->qc_next) {
-        LOCK_QC_EX(q);
         /* Should never fail when holding above locks */
         if (NULL != namename(q->qc_name_n, zone_n)) {
-            free_query_chain_structure(q);
+            clear_query_chain_structure(q);
             if (pol_entry->exp_ttl > 0)
                 q->qc_ttl_x = pol_entry->exp_ttl;
         }
-        UNLOCK_QC(q);    
     }
     *pol = (val_policy_handle_t *) MALLOC (sizeof(val_policy_handle_t));
     if (*pol == NULL) {
@@ -2735,12 +2673,10 @@ val_remove_valpolicy(val_context_t *context, val_policy_handle_t *pol)
 
     /* Flush queries that match this name */
     for(q=ctx->q_list; q; q=q->qc_next) {
-        LOCK_QC_EX(q);
         /* Should never fail when holding above locks */
         if (NULL != namename(q->qc_name_n, p->zone_n)) {
-            free_query_chain_structure(q);
+            clear_query_chain_structure(q);
         }
-        UNLOCK_QC(q);    
     }
     
     /* free the policy */
