@@ -15,7 +15,7 @@
  * WITH THE USE OR PERFORMANCE OF THE SOFTWARE.
  */
 /*
- * Copyright 2005-2009 SPARTA, Inc.  All rights reserved.
+ * Copyright 2005-2011 SPARTA, Inc.  All rights reserved.
  * See the COPYING file distributed with this software for details.
  */
 /*
@@ -70,6 +70,18 @@
 	}\
 } while (0)
 
+void val_res_cancel(struct val_query_chain *matched_q);
+void val_res_nsfallback(struct val_query_chain *matched_q, const char *name_p,
+                        struct timeval *closest_event);
+int _process_rcvd_response(val_context_t * context,
+                           struct queries_for_query *matched_qfq,
+                           struct domain_info **response,
+                           struct queries_for_query **queries,
+                           struct timeval *closest_event,
+                           const char *name_p,
+                           struct name_server *server,
+                           u_char *response_data, size_t response_length);
+
 /*
  * create a name_server struct from the given address rdata
  */
@@ -99,10 +111,9 @@ extract_glue_from_rdata(struct val_rr_rec *addr_rr, struct name_server *ns)
             continue;
         }
 
-        CREATE_NSADDR_ARRAY(new_addr, ns->ns_number_of_addresses + 1);
-        if (new_addr == NULL) {
+        new_addr = create_nsaddr_array(ns->ns_number_of_addresses + 1);
+        if (new_addr == NULL)
             return VAL_OUT_OF_MEMORY;
-        }
 
         for (i = 0; i < ns->ns_number_of_addresses; i++) {
             memcpy(new_addr[i], ns->ns_address[i],
@@ -404,7 +415,7 @@ fix_glue(val_context_t * context,
             if (next_q->qfq_query->qc_state != Q_INIT &&
                        next_q->qfq_query->qc_state != Q_WAIT_FOR_GLUE) {
                 val_log(context, LOG_DEBUG,
-                        "fix_glue(): {%s %d %d} referral failed (%d)", name_p,
+                        "fix_glue(): {%s %d %d} referral/alias processing failed (%d)", name_p,
                         next_q->qfq_query->qc_class_h, next_q->qfq_query->qc_type_h,
                         next_q->qfq_query->qc_state);
             }
@@ -1076,7 +1087,7 @@ follow_referral_or_alias_link(val_context_t * context,
          * is different from the zone we're at
          * ensure that we reset the VAL_QUERY_NO_EDNS0 status
          */
-        if (tp && (matched_qfq->qfq_flags & VAL_QUERY_NO_EDNS0)) {
+        if (tp && (matched_q->qc_flags & VAL_QUERY_NO_EDNS0)) {
             u_char *tp_ref = NULL;
             if (VAL_NO_ERROR != (ret_val = 
                 find_trust_point(context, referral_zone_n, 
@@ -1088,7 +1099,7 @@ follow_referral_or_alias_link(val_context_t * context,
             if (tp_ref && namecmp(tp, tp_ref)) { 
                 val_log(context, LOG_DEBUG, 
                         "follow_referral_or_alias_link(): Re-enabling EDNS0 where previously disabled");
-                matched_qfq->qfq_flags ^= VAL_QUERY_NO_EDNS0;
+                matched_q->qc_flags ^= VAL_QUERY_NO_EDNS0;
             }
             if (tp_ref) {
                 FREE(tp_ref);
@@ -1150,6 +1161,7 @@ follow_referral_or_alias_link(val_context_t * context,
     do {                                                                \
         struct rrset_rec *rr_set;                                       \
         int ret_val;                                                    \
+        u_char *r;                                                      \
         rr_set = find_rr_set (respondent_server, listtype, name_n, type_h, \
                               set_type_h, class_h, ttl_h, hptr, rdata,  \
                               from_section, authoritive, zonecut_n);    \
@@ -1158,11 +1170,23 @@ follow_referral_or_alias_link(val_context_t * context,
         }                                                               \
         else {                                                          \
             if (type_h != ns_t_rrsig) {                                 \
-                /* Add this record to its chain of val_rr_rec's. */         \
+                /* Add this record to its chain of val_rr_rec's. */     \
                 ret_val = add_to_set(rr_set,rdata_len_h,rdata);         \
-            } else  {                                                   \
-                /* Add this record to the sig of rrset_rec. */          \
-                ret_val = add_as_sig(rr_set,rdata_len_h,rdata);         \
+            } else if (VAL_NO_ERROR ==                                  \
+                    (ret_val = add_as_sig(rr_set,rdata_len_h,rdata))) { \
+                /* Add this record to the sig of rrset_rec (above). */  \
+                /* and override the zonecut using the rrsig info */     \
+                if (rr_set->rrs_zonecut_n) {                            \
+                    FREE(rr_set->rrs_zonecut_n);                        \
+                    rr_set->rrs_zonecut_n = NULL;                       \
+                }                                                       \
+                if ((rdata_len_h > SIGNBY) &&                           \
+                     NULL != (r = namename(name_n, &rdata[SIGNBY]))) {  \
+                    rr_set->rrs_zonecut_n =                             \
+                        (u_char *) MALLOC (sizeof(u_char) *             \
+                                wire_name_length(r));                    \
+                    memcpy(rr_set->rrs_zonecut_n, r, wire_name_length(r));\
+                }                                                       \
             }                                                           \
         }                                                               \
         if (ret_val != VAL_NO_ERROR) {                                  \
@@ -1215,9 +1239,6 @@ process_cname_dname_responses(u_char *name_n,
     
     if (type_h == ns_t_cname &&
         matched_q->qc_type_h != ns_t_cname &&
-        matched_q->qc_type_h != ns_t_soa &&
-        matched_q->qc_type_h != ns_t_rrsig &&
-        matched_q->qc_type_h != ns_t_any &&
         namecmp((*qnames)->qnc_name_n, name_n) == 0) {
         
         /*
@@ -1246,10 +1267,7 @@ process_cname_dname_responses(u_char *name_n,
     u_char  *qname_n = (*qnames)->qnc_name_n;
     if (type_h == ns_t_dname &&
         matched_q->qc_type_h != ns_t_dname &&
-        matched_q->qc_type_h != ns_t_soa &&
-        ((matched_q->qc_type_h != ns_t_any &&
-         matched_q->qc_type_h != ns_t_rrsig)||
-        namecmp(qname_n, name_n) != 0) &&
+        namecmp(qname_n, name_n) != 0 &&
         NULL != (p = namename(qname_n, name_n)) &&
         p > qname_n) {
 
@@ -1372,6 +1390,47 @@ prepare_empty_nonexistence(struct rrset_rec **answers,
     return VAL_NO_ERROR;
 }
 
+static void consume_referral_data(struct delegation_info **qc_referral, 
+                                  struct domain_info *di_response,
+                                  struct qname_chain **qnames) {
+
+    if (di_response == NULL || qc_referral == NULL || 
+            *qc_referral == NULL || qnames == NULL)
+        return;
+
+    /*
+     * Consume answers
+     */
+    merge_rrset_recs(&((*qc_referral)->answers), di_response->di_answers);
+    di_response->di_answers = (*qc_referral)->answers;
+    (*qc_referral)->answers = NULL;
+
+    /*
+     * Consume proofs
+     */
+    merge_rrset_recs(&((*qc_referral)->proofs), di_response->di_proofs);
+    di_response->di_proofs = (*qc_referral)->proofs;
+    (*qc_referral)->proofs = NULL;
+
+    /*
+     * Consume qnames
+     */
+    if (*qnames == NULL)
+        *qnames = (*qc_referral)->qnames;
+    else if ((*qc_referral)->qnames) {
+        struct qname_chain *t_q;
+        for (t_q = *qnames; t_q->qnc_next; t_q = t_q->qnc_next);
+            t_q->qnc_next = (*qc_referral)->qnames;
+    }
+    (*qc_referral)->qnames = NULL;
+
+    /*
+     * Note that we don't free qc_referral here 
+     */
+    free_referral_members(*qc_referral);
+    qc_referral = NULL;
+}
+
 /*
  * The main routine for processing response data
  *  
@@ -1544,6 +1603,13 @@ digest_response(val_context_t * context,
         ret_val = prepare_empty_nonexistence(&di_response->di_proofs, 
                         matched_q->qc_respondent_server,
                         query_name_n, query_type_h, query_class_h, hptr);
+        /*
+         * copy any answers that may be present in the referral structure
+         * E.g. if this response was returned after following a CNAME/DNAME
+         */
+        if (matched_q->qc_referral != NULL) {
+            consume_referral_data(&matched_q->qc_referral, di_response, qnames);
+        } 
         goto done; 
     }
 
@@ -1602,6 +1668,20 @@ digest_response(val_context_t * context,
                 (query_type_h == ns_t_any) ||
                 (query_type_h == ns_t_rrsig)) {
 
+                nothing_other_than_alias = 0;
+            } else if (ALIAS_MATCH_TYPE(query_type_h)){
+                int referral_error = 0;
+                /* process CNAMEs or DNAMEs if they exist */
+                if ((VAL_NO_ERROR != (ret_val = 
+                        process_cname_dname_responses(name_n, type_h, rdata, 
+                                                  matched_q, qnames, 
+                                                  &referral_error))) || 
+                        (referral_error)) {
+                    if (referral_error) 
+                        val_log(context, LOG_DEBUG, "digest_response(): CNAME/DNAME error or loop encountered");
+                    goto done;
+                }
+            } else {
                 nothing_other_than_alias = 0;
             }
         }
@@ -1619,20 +1699,6 @@ digest_response(val_context_t * context,
          * If it is from the additional section it may contain some DNSSEC meta-data or it may be glue
          */
         if (from_section == VAL_FROM_ANSWER) {
-            int referral_error = 0;
-
-            /* process CNAMEs or DNAMEs if they exist */
-            if ((VAL_NO_ERROR != (ret_val = 
-                    process_cname_dname_responses(name_n, type_h, rdata, 
-                                                  matched_q, qnames, 
-                                                  &referral_error))) || 
-                    (referral_error)) {
-                if (referral_error) 
-                    val_log(context, LOG_DEBUG, "digest_response(): CNAME/DNAME error or loop encountered");
-
-                goto done;
-            }
-
             SAVE_RR_TO_LIST(matched_q->qc_respondent_server, 
                             &learned_answers, name_n, type_h,
                             set_type_h, class_h, ttl_h, hptr, rdata,
@@ -1684,6 +1750,32 @@ digest_response(val_context_t * context,
             }
         }
 
+        /*
+         * If we're asking for a DS and have got an SOA with the same name
+         * the name server likely does not understand DNSSEC
+         */
+        if ( query_type_h == ns_t_ds &&
+             set_type_h == ns_t_soa &&
+             !namecmp(name_n, query_name_n)) {
+            val_log(context, LOG_DEBUG, "digest_response(): bad response for DS record. NS probably not DNSSEC-capable.");
+            matched_q->qc_state = Q_WRONG_ANSWER;
+            ret_val = VAL_NO_ERROR;
+            goto done;
+        }
+        /*
+         * If we've received a CNAME/DNAME response for a type that cannot be followed using cnames/dnames
+         * we're not going to follow it
+         */
+        if ((set_type_h == ns_t_cname || set_type_h == ns_t_dname) &&
+             !ALIAS_MATCH_TYPE(query_type_h)) {
+            val_log(context, LOG_DEBUG, 
+                    "digest_response(): Won't follow alias for type %d.", 
+                    query_type_h);
+            matched_q->qc_state = Q_WRONG_ANSWER;
+            ret_val = VAL_NO_ERROR;
+            goto done;
+        }
+
         /*
          * if we have an SOA in the ans/auth section
          *  or if we have a DNSKEY in the ans section
@@ -1735,6 +1827,10 @@ digest_response(val_context_t * context,
             if (query_type_h == ns_t_ds &&
                 NULL != namename (name_n, query_name_n)) {
                 fix_zonecut = 0;
+                val_log(context, LOG_DEBUG, "digest_response(): bad response for DS record. NS probably not DNSSEC-capable.");
+                matched_q->qc_state = Q_WRONG_ANSWER;
+                ret_val = VAL_NO_ERROR;
+                goto done;
             } else if (nothing_other_than_alias) {
                 /* 
                  * make sure that the NS is closer to the alias than the target; 
@@ -1759,43 +1855,48 @@ digest_response(val_context_t * context,
                if (!n1 || !n2 || namename(n1, n2))
                    fix_zonecut = 0;
             } 
-
             if (fix_zonecut && !zonecut_was_modified) {
 
                 zonecut_was_modified = 1;
-                /* update the zonecut information */
-                if (matched_q->qc_zonecut_n)
-                    FREE(matched_q->qc_zonecut_n);
-                len = wire_name_length(name_n);
-                matched_q->qc_zonecut_n = 
-                    (u_char *) MALLOC (len * sizeof(u_char));
-                if (matched_q->qc_zonecut_n == NULL)
-                    goto done;    
-                memcpy (matched_q->qc_zonecut_n, name_n, len);
-                rrs_zonecut_n = matched_q->qc_zonecut_n;
-    
-                if (rrs_zonecut_n && 
-                        ns_name_ntop(rrs_zonecut_n, rrs_zonecut_p, sizeof(rrs_zonecut_p)) == -1) {
-                    ret_val =  VAL_BAD_ARGUMENT;
-                    goto done;
-                }
 
-                val_log(context, LOG_DEBUG, 
+                /* don't do anything if our existing zonecut matches the new one  */
+                if (!matched_q->qc_zonecut_n ||
+                    namecmp(matched_q->qc_zonecut_n, name_n)) {
+
+                    /* update the zonecut information */
+                    if (matched_q->qc_zonecut_n) 
+                        FREE(matched_q->qc_zonecut_n);
+                    len = wire_name_length(name_n);
+                    matched_q->qc_zonecut_n = 
+                        (u_char *) MALLOC (len * sizeof(u_char));
+                    if (matched_q->qc_zonecut_n == NULL)
+                        goto done;    
+                    memcpy (matched_q->qc_zonecut_n, name_n, len);
+                    rrs_zonecut_n = matched_q->qc_zonecut_n;
+    
+                    if (rrs_zonecut_n && 
+                            ns_name_ntop(rrs_zonecut_n, rrs_zonecut_p, sizeof(rrs_zonecut_p)) == -1) {
+                        ret_val =  VAL_BAD_ARGUMENT;
+                        goto done;
+                    }
+
+                    val_log(context, LOG_DEBUG, 
                         "digest_response(): Setting zonecut for {%s %d %d} query responses to %s",
                         query_name_p, query_class_h, query_type_h, rrs_zonecut_p);
-                /*
-                 * go back to all the rrsets that we created 
-                 * and fix the zonecut info 
-                 */
-                FIX_ZONECUT(learned_answers, rrs_zonecut_n, ret_val);
-                if (ret_val != VAL_NO_ERROR)
-                    goto done;
-                FIX_ZONECUT(learned_proofs, rrs_zonecut_n, ret_val);
-                if (ret_val != VAL_NO_ERROR)
-                    goto done;
-                FIX_ZONECUT(learned_zones, rrs_zonecut_n, ret_val);
-                if (ret_val != VAL_NO_ERROR)
-                    goto done;
+                    /*
+                     * go back to all the rrsets that we created 
+                     * and fix the zonecut info 
+                     */
+                    FIX_ZONECUT(learned_answers, rrs_zonecut_n, ret_val);
+                    if (ret_val != VAL_NO_ERROR)
+                        goto done;
+                    FIX_ZONECUT(learned_proofs, rrs_zonecut_n, ret_val);
+                    if (ret_val != VAL_NO_ERROR)
+                        goto done;
+                    FIX_ZONECUT(learned_zones, rrs_zonecut_n, ret_val);
+                    if (ret_val != VAL_NO_ERROR)
+                        goto done;
+                }
             } else if (fix_zonecut && namecmp(rrs_zonecut_n, name_n)) {
                 /*
                  * Multiple Zonecuts
@@ -1926,37 +2027,7 @@ digest_response(val_context_t * context,
          * Check if this is the response to a referral request 
          */
         if (matched_q->qc_referral != NULL) {
-
-            /*
-             * Consume answers
-             */
-            merge_rrset_recs(&matched_q->qc_referral->answers, di_response->di_answers);
-            di_response->di_answers = matched_q->qc_referral->answers;
-            matched_q->qc_referral->answers = NULL;
-
-            /*
-             * Consume proofs
-             */
-            merge_rrset_recs(&matched_q->qc_referral->proofs, di_response->di_proofs);
-            di_response->di_proofs = matched_q->qc_referral->proofs;
-            matched_q->qc_referral->proofs = NULL;
-
-            /*
-             * Consume qnames
-             */
-            if (*qnames == NULL)
-                *qnames = matched_q->qc_referral->qnames;
-            else if (matched_q->qc_referral->qnames) {
-                struct qname_chain *t_q;
-                for (t_q = *qnames; t_q->qnc_next; t_q = t_q->qnc_next);
-                t_q->qnc_next = matched_q->qc_referral->qnames;
-            }
-            matched_q->qc_referral->qnames = NULL;
-
-            /*
-             * Note that we don't free qc_referral here 
-             */
-            free_referral_members(matched_q->qc_referral);
+            consume_referral_data(&matched_q->qc_referral, di_response, qnames);
         } 
 
         matched_q->qc_state = Q_ANSWERED;
@@ -2015,7 +2086,11 @@ val_resquery_send(val_context_t * context,
      */
     struct name_server *nslist;
     if ((matched_qfq == NULL) || 
-            (matched_qfq->qfq_query->qc_ns_list == NULL)) {
+        (matched_qfq->qfq_query->qc_ns_list == NULL)
+#ifndef VAL_NO_ASYNC
+        || (matched_qfq->qfq_flags & VAL_QUERY_ASYNC)
+#endif
+        ) {
         return VAL_BAD_ARGUMENT;
     }
     matched_q = matched_qfq->qfq_query; /* Can never be NULL if matched_qfq is not NULL */
@@ -2064,7 +2139,12 @@ val_resquery_rcv(val_context_t * context,
 
     int             ret_val;
 
-    if ((matched_qfq == NULL) || (response == NULL) || (queries == NULL) || (pending_desc == NULL))
+    if ((matched_qfq == NULL) || (response == NULL) || (queries == NULL) ||
+        (pending_desc == NULL)
+#ifndef VAL_NO_ASYNC
+        || (matched_qfq->qfq_flags & VAL_QUERY_ASYNC)
+#endif
+        )
         return VAL_BAD_ARGUMENT;
 
     matched_q = matched_qfq->qfq_query; /* Can never be NULL if matched_qfq is not NULL */
@@ -2075,23 +2155,63 @@ val_resquery_rcv(val_context_t * context,
     if (ret_val == SR_NO_ANSWER_YET)
         return VAL_NO_ERROR;
 
-    matched_q->qc_respondent_server = server;
-
+    /** convert name to printable string */
     if (ns_name_ntop(matched_q->qc_name_n, name_p, sizeof(name_p)) == -1) {
         matched_q->qc_state = Q_RESPONSE_ERROR;
         if (response_data)
             FREE(response_data);
         return VAL_NO_ERROR;
     }
+
+    /** if there was no answer, try smaller edns0 size */
     if (ret_val == SR_NO_ANSWER) {
-        int edns0;
-        if (-1 == res_nsfallback(matched_q->qc_trans_id, closest_event,
+        val_res_nsfallback(matched_q, name_p, closest_event);
+        if (response_data)
+            FREE(response_data);
+        return VAL_NO_ERROR;
+    }
+
+    ret_val = _process_rcvd_response(context, matched_qfq, response, queries,
+                                     closest_event, name_p, server,
+                                     response_data, response_length);
+
+    return ret_val;
+}
+
+void
+val_res_cancel(struct val_query_chain *matched_q)
+{
+#ifndef VAL_NO_ASYNC
+    if (matched_q->qc_ea) {
+        res_async_query_free(matched_q->qc_ea);
+        matched_q->qc_ea = NULL;
+    }
+    else
+#endif
+        res_cancel(&(matched_q->qc_trans_id));
+}
+
+void
+val_res_nsfallback(struct val_query_chain *matched_q, const char *name_p,
+                   struct timeval *closest_event)
+{
+    int edns0, ret_val;
+
+#ifndef VAL_NO_ASYNC
+    if (matched_q->qc_ea)
+        ret_val = res_nsfallback_ea(matched_q->qc_ea, closest_event,
+                                    name_p, matched_q->qc_class_h, 
+                                    matched_q->qc_type_h, &edns0);
+    else
+#endif
+        ret_val = res_nsfallback(matched_q->qc_trans_id, closest_event,
                                  name_p, matched_q->qc_class_h, 
-                                 matched_q->qc_type_h, &edns0)) {
-            /* give up */
-            res_cancel(&(matched_q->qc_trans_id));
-            matched_q->qc_state = Q_RESPONSE_ERROR;
-        }
+                                 matched_q->qc_type_h, &edns0);
+    if (-1 == ret_val) {
+        matched_q->qc_state = Q_RESPONSE_ERROR;
+        val_res_cancel(matched_q);
+    }
+    else {
         if (edns0) {
             /* reset the flag if enabled */
             if(matched_q->qc_flags & VAL_QUERY_NO_EDNS0)
@@ -2101,10 +2221,23 @@ val_resquery_rcv(val_context_t * context,
             if (!(matched_q->qc_flags & VAL_QUERY_NO_EDNS0))
                 matched_q->qc_flags |= VAL_QUERY_NO_EDNS0;
         }
-        if (response_data)
-            FREE(response_data);
-        return VAL_NO_ERROR;
     }
+}
+
+int
+_process_rcvd_response(val_context_t * context,
+                       struct queries_for_query *matched_qfq,
+                       struct domain_info **response,
+                       struct queries_for_query **queries,
+                       struct timeval *closest_event,
+                       const char *name_p,
+                       struct name_server *server,
+                       u_char *response_data, size_t response_length)
+{
+    struct val_query_chain *matched_q = matched_qfq->qfq_query;
+    int ret_val;
+
+    matched_q->qc_respondent_server = server;
 
     *response = (struct domain_info *) MALLOC(sizeof(struct domain_info));
     if (*response == NULL) {
@@ -2142,33 +2275,19 @@ val_resquery_rcv(val_context_t * context,
         return ret_val;
     }
 
-    if (matched_q->qc_state > Q_ERROR_BASE) {
+    if (matched_q->qc_state == Q_RESPONSE_ERROR) {
         /* try a different NS if possible */
         int edns0;
         free_domain_info_ptrs(*response);
         FREE(*response);
         *response = NULL;
-        if (-1 == res_nsfallback(matched_q->qc_trans_id, closest_event,
-                                 name_p, matched_q->qc_class_h, 
-                                 matched_q->qc_type_h, &edns0)) {
-            res_cancel(&(matched_q->qc_trans_id));
-            matched_q->qc_state = Q_RESPONSE_ERROR;
-        } else {
-            if (edns0) {
-                /* reset the flag if enabled */
-                if(matched_q->qc_flags & VAL_QUERY_NO_EDNS0)
-                    matched_q->qc_flags ^= VAL_QUERY_NO_EDNS0;
-            } else {
-                /* set the flag if disabled */
-                if (!(matched_q->qc_flags & VAL_QUERY_NO_EDNS0))
-                    matched_q->qc_flags |= VAL_QUERY_NO_EDNS0;
-            }
+        val_res_nsfallback(matched_q, name_p, closest_event);
+        if (matched_q->qc_state != Q_RESPONSE_ERROR)
             matched_q->qc_state = Q_SENT;
-        }
     }
     else {
         /* we're good to go, cancel pending query transactions */
-        res_cancel(&(matched_q->qc_trans_id));
+        val_res_cancel(matched_q);
         (*response)->di_res_error = SR_UNSET;
     }
 
@@ -2176,11 +2295,145 @@ val_resquery_rcv(val_context_t * context,
 
     /*
      * What happens when an empty NXDOMAIN is returned? 
-     */
-    /*
      * What happens when an empty NOERROR is returned? 
      */
 
     return VAL_NO_ERROR;
 }
 
+/*****************************************************************************
+ *
+ *
+ * Asynchronous API
+ *
+ *
+ ****************************************************************************/
+#ifndef VAL_NO_ASYNC
+
+
+/*
+ * This is the interface between libval and libsres for sending queries
+ * Get a (set of) answer(s) from the default NS's.
+ */
+int
+val_resquery_async_send(val_context_t * context,
+                        struct queries_for_query *matched_qfq)
+{
+    char            name_p[NS_MAXDNAME];
+    char            name_buf[INET6_ADDRSTRLEN + 1];
+    struct val_query_chain *matched_q;
+
+    if ((matched_qfq == NULL) || (matched_qfq->qfq_query->qc_ns_list == NULL))
+        return VAL_BAD_ARGUMENT;
+
+    /** Can never be NULL if matched_qfq is not NULL */
+    matched_q = matched_qfq->qfq_query;
+
+    if (ns_name_ntop(matched_q->qc_name_n, name_p, sizeof(name_p)) == -1)
+        return VAL_BAD_ARGUMENT;
+
+    if (val_log_debug_level() >= LOG_DEBUG ) {
+        struct name_server *tempns;
+        struct name_server *nslist = matched_q->qc_ns_list;
+
+        val_log(context, LOG_DEBUG,
+                "val_resquery_async_send(): Sending query for {%s %d %d} to:", 
+                name_p, matched_q->qc_class_h, matched_q->qc_type_h);
+        for (tempns = nslist; tempns; tempns = tempns->ns_next) {
+            val_log(context, LOG_DEBUG, "    %s",
+                    val_get_ns_string((struct sockaddr *)tempns->ns_address[0],
+                                      name_buf, sizeof(name_buf)));
+        }
+    }
+
+    matched_q->qc_ea = res_async_query_send(name_p, matched_q->qc_type_h,
+                                            matched_q->qc_class_h, 
+                                            matched_q->qc_ns_list);
+    if (!matched_q->qc_ea)
+        matched_q->qc_state = Q_QUERY_ERROR;
+
+    return VAL_NO_ERROR;
+}
+
+/*
+ * This is the interface between libval and libsres for receiving responses 
+ */
+int
+val_resquery_async_rcv(val_context_t * context,
+                       struct queries_for_query *matched_qfq,
+                       struct domain_info **response,
+                       struct queries_for_query **queries,
+                       fd_set *pending_desc,
+                       struct timeval *closest_event)
+{
+    struct name_server *server = NULL;
+    u_char       *response_data = NULL;
+    size_t       response_length = 0;
+    char         name_p[NS_MAXDNAME], stream;
+    struct val_query_chain *matched_q;
+    int             ret_val, handled;
+
+    if ((matched_qfq == NULL) || (response == NULL) || (queries == NULL) ||
+        (pending_desc == NULL))
+        return VAL_BAD_ARGUMENT;
+
+    matched_q = matched_qfq->qfq_query; /* ! NULL if matched_qfq ! NULL */
+    *response = NULL;
+    stream = res_async_ea_is_using_stream(matched_q->qc_ea);
+
+    /** check for a response */
+    ret_val = res_async_query_handle(matched_q->qc_ea, &handled, pending_desc);
+    if (ret_val == SR_NO_ANSWER_YET)
+        return VAL_NO_ERROR;
+
+    /** get the response, if any */
+    ret_val = res_io_get_a_response(matched_q->qc_ea, &response_data,
+                                    &response_length, &server);
+
+    /** convert name to printable string */
+    if (ns_name_ntop(matched_q->qc_name_n, name_p, sizeof(name_p)) == -1) {
+        matched_q->qc_state = Q_RESPONSE_ERROR;
+        if (response_data)
+            FREE(response_data);
+        return VAL_NO_ERROR;
+    }
+
+    /** if there was no answer, try smaller edns0 size */
+    if (ret_val == SR_NO_ANSWER) {
+        if (stream || !res_async_ea_is_using_stream(matched_q->qc_ea))
+            val_res_nsfallback(matched_q, name_p, closest_event);
+        if (response_data)
+            FREE(response_data);
+        return VAL_NO_ERROR;
+    }
+
+    ret_val = _process_rcvd_response(context, matched_qfq, response, queries,
+                                     closest_event, name_p, server,
+                                     response_data, response_length);
+
+    return ret_val;
+ }
+ 
+int
+val_async_select_info(val_context_t *context, fd_set *activefds,
+                      int *nfds, struct timeval *timeout)
+{
+    val_async_status *as;
+    struct queries_for_query *qfq;
+
+    if (NULL == context)
+        return VAL_BAD_ARGUMENT;
+
+    for (as = context->as_list; as; as = as->val_as_next)
+        for (qfq = as->val_as_queries; qfq; qfq = qfq->qfq_next) {
+            if (!qfq->qfq_query->qc_ea)
+                continue;
+            res_async_query_select_info(qfq->qfq_query->qc_ea, nfds, activefds,
+                                        timeout);
+        }
+
+    return VAL_NO_ERROR;
+}
+
+
+#endif /* VAL_NO_ASYNC */
