@@ -1,10 +1,16 @@
 /*
- * Copyright 2005-2009 SPARTA, Inc.  All rights reserved.
+ * Copyright 2005-2011 SPARTA, Inc.  All rights reserved.
  * See the COPYING file distributed with this software for details.
  */
 #ifndef VAL_RESQUERY_H
 #define VAL_RESQUERY_H
 
+/* alias cannot match the following types */
+#define ALIAS_MATCH_TYPE(type_h) ((type_h != ns_t_any &&    \
+                                   type_h != ns_t_rrsig &&  \
+                                   type_h != ns_t_dnskey && \
+                                   type_h != ns_t_ds) ? 1 : 0)
+
 struct glue_fetch_bucket {
     struct queries_for_query *qfq[MAX_GLUE_FETCH_DEPTH];
     int qfq_count;
@@ -44,4 +50,16 @@ int             val_resquery_rcv(val_context_t * context,
                                  fd_set *pending_desc,
                                  struct timeval *closest_event);
 
+#ifndef VAL_NO_ASYNC
+
+int             val_resquery_async_send(val_context_t * context,
+                                        struct queries_for_query *matched_qfq);
+int             val_resquery_async_rcv(val_context_t * context,
+                                       struct queries_for_query *matched_qfq,
+                                       struct domain_info **response,
+                                       struct queries_for_query **queries,
+                                       fd_set *pending_desc,
+                                       struct timeval *closest_event);
+#endif /* VAL_NO_ASYNC */
+
 #endif                          /* VAL_RESQUERY_H */
