@@ -16,7 +16,7 @@
  * WITH THE USE OR PERFORMANCE OF THE SOFTWARE.
  */
 /*
- * Copyright 2005-2009 SPARTA, Inc.  All rights reserved.
+ * Copyright 2005-2011 SPARTA, Inc.  All rights reserved.
  * See the COPYING file distributed with this software for details.
  */
 #include "validator-config.h"
@@ -28,13 +28,12 @@
 #include <sys/time.h>
 #include <sys/types.h>
 
-#ifndef NAMESER_HAS_HEADER
+#include <arpa/nameser.h>
 #ifdef HAVE_ARPA_NAMESER_COMPAT_H
 #include <arpa/nameser_compat.h>
 #else
 #include "arpa/header.h"
 #endif
-#endif                          /* NAMESER_HAS_HEADER */
 
 #include <validator/resolver.h>
 #include <validator/validator.h>
@@ -518,7 +517,7 @@ is_tail(u_char * full, u_char * tail)
 
     if (f_len == t_len) {
         if (f_len)
-            return memcmp(full, tail, f_len) == 0;
+            return namecmp(full, tail) == 0;
         else
             return 0;
     }
@@ -526,7 +525,7 @@ is_tail(u_char * full, u_char * tail)
     if (t_len > f_len)
         return FALSE;
 
-    if (memcmp(&full[f_len - t_len], tail, t_len) == 0) {
+    if (namecmp(&full[f_len - t_len], tail) == 0) {
         size_t        index = 0;
 
         while (index < (f_len - t_len)) {
@@ -557,7 +556,7 @@ add_to_set(struct rrset_rec *rr_set, size_t rdata_len_h,
 
     rr->rr_rdata = (u_char *) MALLOC(rdata_len_h * sizeof(u_char));
     if (rr->rr_rdata == NULL) {
-        free(rr);
+        FREE(rr);
         return VAL_OUT_OF_MEMORY;
     }
 
@@ -604,7 +603,7 @@ add_as_sig(struct rrset_rec *rr_set, size_t rdata_len_h,
 
     rr->rr_rdata = (u_char *) MALLOC(rdata_len_h * sizeof(u_char));
     if (rr->rr_rdata == NULL) {
-        free(rr);
+        FREE(rr);
         return VAL_OUT_OF_MEMORY;
     }
 
@@ -732,7 +731,7 @@ init_rr_set(struct rrset_rec *new_set, u_char * name_n,
         (s != ns_t_nsec &&                    /* If the type is not nxt: */    \
         a->rrs_type_h == s &&                    /* does type match */        \
         a->rrs_class_h == c &&                   /* does class match */       \
-        memcmp (a->rrs_name_n,n,l)==0            /* does name match */        \
+        namecmp (a->rrs_name_n,n)==0            /* does name match */        \
         )                                                                     \
         ||                                   /* or */                         \
         (s == ns_t_nsec &&														\
@@ -740,7 +739,7 @@ init_rr_set(struct rrset_rec *new_set, u_char * name_n,
         a->rrs_data!=NULL &&                     /* is there data here */     \
         a->rrs_class_h == c &&                   /* does class match */       \
 		a->rrs_type_h == ns_t_nsec &&													\
-        memcmp (a->rrs_name_n,n,l)==0 &&         /* does name match */        \
+        namecmp (a->rrs_name_n,n)==0 &&         /* does name match */        \
         is_tail(a->rrs_data->rr_rdata,&r[SIGNBY])                               \
                                                  /* does sig match nxt */     \
         )                                                                     \
@@ -750,7 +749,7 @@ init_rr_set(struct rrset_rec *new_set, u_char * name_n,
         a->rrs_sig!=NULL &&                      /* is there a sig here */    \
         a->rrs_class_h == c &&                   /* does class match */       \
 		a->rrs_type_h == ns_t_nsec &&													\
-        memcmp (a->rrs_name_n,n,l)==0 &&         /* does name match */        \
+        namecmp (a->rrs_name_n,n)==0 &&         /* does name match */        \
         is_tail(r,&a->rrs_sig->rr_rdata[SIGNBY])                                \
                                                  /* does sig match nxt */     \
         )                                                                     \
@@ -872,8 +871,9 @@ decompress(u_char ** rdata,
     size_t        other_name_length = 0;
     size_t        name_length = 0;
     size_t        insert_index = 0;
-    size_t        working_increment;
+    int           working_increment;
     size_t        expansion = 0;
+    size_t        len = 0;
 
     if ((rdata == NULL) || (response == NULL) || (rdata_len_h == NULL))
         return VAL_BAD_ARGUMENT;
@@ -903,11 +903,16 @@ decompress(u_char ** rdata,
     case ns_t_nsec:
     default:
         new_size = (size_t) * rdata_len_h;
+        if (new_size == 0)
+            return VAL_NO_ERROR;
+
         *rdata = (u_char *) MALLOC(new_size * sizeof(u_char));
         if (*rdata == NULL)
             return VAL_OUT_OF_MEMORY;
 
-        memcpy(&(*rdata)[insert_index], &response[rdata_index], new_size);
+        if (response + working_index + new_size > end)
+            return VAL_BAD_ARGUMENT;
+        memcpy(&(*rdata)[insert_index], &response[working_index], new_size);
         *rdata_len_h = *rdata_len_h;    /* No change */
         break;
 
@@ -980,8 +985,10 @@ decompress(u_char ** rdata,
          * Copy any remaining data 
          */
 
-        memcpy(&(*rdata)[insert_index], &response[working_index],
-               *rdata_len_h + expansion - other_name_length - name_length);
+        len = *rdata_len_h + expansion - other_name_length - name_length;
+        if (response + working_index + len  > end)
+            return VAL_BAD_ARGUMENT;
+        memcpy(&(*rdata)[insert_index], &response[working_index], len);
 
         *rdata_len_h += expansion;
 
@@ -992,6 +999,8 @@ decompress(u_char ** rdata,
          */
     case ns_t_srv:
 
+        if (response + working_index + (2 * sizeof(u_int16_t)) > end)
+            return VAL_BAD_ARGUMENT;
         memcpy(&prefix[p_index], &response[working_index],
                2 * sizeof(u_int16_t));
         working_index += 2 * sizeof(u_int16_t);
@@ -1006,6 +1015,8 @@ decompress(u_char ** rdata,
     case ns_t_kx:
     case ns_t_px:
 
+        if (response + working_index + sizeof(u_int16_t) > end)
+            return VAL_BAD_ARGUMENT;
         memcpy(&prefix[p_index], &response[working_index],
                sizeof(u_int16_t));
         working_index += sizeof(u_int16_t);
@@ -1093,6 +1104,8 @@ decompress(u_char ** rdata,
         if (*rdata == NULL)
             return VAL_OUT_OF_MEMORY;
 
+        if (response + working_index + SIGNBY > end)
+            return VAL_BAD_ARGUMENT;
         memcpy(&(*rdata)[insert_index], &response[working_index], SIGNBY);
         insert_index += SIGNBY;
         working_index += SIGNBY;
@@ -1101,8 +1114,10 @@ decompress(u_char ** rdata,
         insert_index += name_length;
         working_index += working_increment;
 
-        memcpy(&(*rdata)[insert_index], &response[working_index],
-               *rdata_len_h - working_increment - SIGNBY);
+        len = *rdata_len_h - working_increment - SIGNBY;
+        if (response + working_index + len > end)
+            return VAL_BAD_ARGUMENT;
+        memcpy(&(*rdata)[insert_index], &response[working_index], len);
 
         *rdata_len_h += expansion;
     }
@@ -1141,6 +1156,15 @@ extract_from_rr(u_char * response,
 
     *response_index += ret_val;
 
+    /* check if we have enough data to read the envelope */
+    if (response + *response_index + 
+        sizeof(u_int16_t) + 
+        sizeof(u_int16_t) + 
+        sizeof(u_int32_t) + 
+        sizeof(u_int16_t) > end) {
+            return VAL_BAD_ARGUMENT;
+    }
+
     /*
      * Extract the type, and save it in host format 
      */
@@ -1184,6 +1208,8 @@ extract_from_rr(u_char * response,
         /*
          * Extract the set type, and save it in host format 
          */
+        if (response + *response_index + sizeof(u_int16_t) > end)
+            return VAL_BAD_ARGUMENT;
         memcpy(&net_short, &response[*response_index], sizeof(u_int16_t));
         *set_type_h = ntohs(net_short);
     } else
@@ -1249,6 +1275,7 @@ lower(u_int16_t type_h, u_char * rdata, size_t len)
     case ns_t_a6: 
     case ns_t_naptr:
     case ns_t_nsec: 
+    case ns_t_rrsig: 
     default:
 
         return;
@@ -1315,18 +1342,11 @@ lower(u_int16_t type_h, u_char * rdata, size_t len)
          * The last case is RR's with names in the middle. 
          */
         /*
-         * Note: this code is never used as SIG's are the only record in
-         * this case.  SIG's are not signed, so they never are run through
-         * this code.  This is left here in case other RR's are defined in
+         * Note: this code is never used, since there are currently no 
+         * record types in this category. 
+         * This is left here in case RR's are defined in
          * this unfortunate (for them) manner.
          */
-    case ns_t_rrsig:
-
-        index = SIGNBY;
-
-        lower_name(rdata, &index);
-
-        return;
     }
 }
 
@@ -1434,8 +1454,10 @@ link_rr(struct val_rr_rec **cs, struct val_rr_rec *cr)
             /*
              * cr is a copy of an existing record, forget it... 
              */
-            FREE(cr->rr_rdata);
-            FREE(cr);
+            if (cr->rr_next) { // old code was freeing w/out checking
+                cr->rr_next = NULL;
+            }
+            res_sq_free_rr_recs(&cr);
             return DUPLICATE;
         } else if (ret_val > 0
                    || (ret_val == 0 && length == cr->rr_rdata_length)) {
@@ -1464,8 +1486,10 @@ link_rr(struct val_rr_rec **cs, struct val_rr_rec *cr)
                     /*
                      * cr is a copy of an existing record, forget it... 
                      */
-                    FREE(cr->rr_rdata);
-                    FREE(cr);
+                    if (cr->rr_next) { // old code was freeing w/out checking
+                        cr->rr_next = NULL;
+                    }
+                    res_sq_free_rr_recs(&cr);
                     return DUPLICATE;
                 } else if (ret_val > 0
                            || (ret_val == 0
