@@ -1,5 +1,5 @@
 /*
- * Copyright 2005-2009 SPARTA, Inc.  All rights reserved.
+ * Copyright 2005-2011 SPARTA, Inc.  All rights reserved.
  * See the COPYING file distributed with this software for details.
  */
 /*
@@ -17,6 +17,7 @@
 #include <time.h>
 #include <ctype.h>
 
+#include <arpa/nameser.h>
 #include <validator/resolver.h>
 #include <validator/validator.h>
 #include <validator/validator-internal.h>
@@ -677,6 +678,7 @@ verify_next_assertion(val_context_t * ctx,
     struct val_rr_rec  *keyrr;
     u_int16_t       tag_h;
     int             is_verified = 0;
+    char            name_p[NS_MAXDNAME];
 
     if ((as == NULL) || (as->val_ac_rrset.ac_data == NULL) || (the_trust == NULL)) {
         val_log(ctx, LOG_INFO, "verify_next_assertion(): Cannot verify assertion - no data");
@@ -686,6 +688,10 @@ verify_next_assertion(val_context_t * ctx,
     the_set = as->val_ac_rrset.ac_data;
     dnskey.public_key = NULL;
 
+
+    if (-1 == ns_name_ntop(the_set->rrs_name_n, name_p, sizeof(name_p)))
+        snprintf(name_p, sizeof(name_p), "unknown/error");
+
     if (the_set->rrs_sig == NULL) {
         val_log(ctx, LOG_INFO, "verify_next_assertion(): RRSIG is missing");
         as->val_ac_status = VAL_AC_RRSIG_MISSING;
@@ -778,6 +784,9 @@ verify_next_assertion(val_context_t * ctx,
 
             // if success, break
             if (is_verified) {
+                val_log(ctx, LOG_INFO, "verify_next_assertion(): Verified a RRSIG for %s (%s) using a DNSKEY (%d)",
+                        name_p, p_type(the_set->rrs_type_h),
+                        dnskey.key_tag);
                 break;
             }
 
@@ -794,8 +803,7 @@ verify_next_assertion(val_context_t * ctx,
 
         if (as->val_ac_status == VAL_AC_TRUST ||
             (is_verified && 
-                    (as->val_ac_status == VAL_AC_TRUST_NOCHK || 
-                     nextrr->rr_status == VAL_AC_TRUST_POINT))) {
+                     nextrr->rr_status == VAL_AC_TRUST_POINT)) {
             /* we've verified a trust anchor */
 
             as->val_ac_status = VAL_AC_TRUST; 
@@ -808,6 +816,7 @@ verify_next_assertion(val_context_t * ctx,
             
         /* else check if we're trying to verify some key in the authentication chain */
         } else if ( the_set->rrs_type_h == ns_t_dnskey && 
+                    as != the_trust &&
                     as->val_ac_status != VAL_AC_TRUST &&
                      (is_verified || the_sig->rr_status == VAL_AC_ALGORITHM_NOT_SUPPORTED)) {
 
